#pragma ident "$Id: ida.h,v 1.10 2004/06/25 18:34:56 dechavez Exp $"
/*======================================================================
 *
 *  Include file of defines and type declarations for routines which
 *  manipulate Project IDA field data.
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Copyright (c) 1997 Regents of the University of California.
 * All rights reserved.
 *====================================================================*/
#ifndef ida_h_included
#define ida_h_included

#include "platform.h"
#include "util.h"
#include "dbio.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

#define IDA_BUFSIZ      1024 /* Standard record length              */
#define IDA_BUFLEN  IDA_BUFSIZ
#define IDA_HEADLEN      64  /* Standard header length              */
#define IDA_SNAMLEN       8  /* Max station name length (inc. NULL) */
#define IDA_CNAMLEN       8  /* Max channel name length (inc. NULL) */
#define IDA_INAMLEN       8  /* Max instrument name length (inc. NULL) */
#define IDA_MNAMLEN      32  /* Max channel map name len.  (inc. NULL) */
#define IDA_MAXLOGLEN   988  /* Max log message  length (inc. NULL) */
#define IDA_MAXDLEN (IDA_BUFSIZ - IDA_HEADLEN) /* Max size of data area */
#define IDA_MAXSTREAMS   64  /* Max number of defined streams       */
#define IDA_MAXCALCHAN   12  /* Max number of calibratable channels */
#define IDA_MAXOLDCNF   225  /* Max dim of old-style config table   */
#define IDA_MAXOLDHREC   10  /* Max number of old-style H rec types */
#define IDA_MAXEVENTS    27  /* Max number of events in E record    */

/*  Hardware system codes  */

#define IDA_MK3  0x0001
#define IDA_MK4  0x0002
#define IDA_MK5  0x0004
#define IDA_MK6A 0x0008
#define IDA_MK6B 0x0010
#define IDA_MK7A 0x0020
#define IDA_MK7B 0x0040

/*  Record types  */

#define IDA_UNKNOWN  0x0000
#define IDA_IDENT    0x0001
#define IDA_DATA     0x0002
#define IDA_CONFIG   0x0004
#define IDA_LOG      0x0008
#define IDA_CALIB    0x0010
#define IDA_EVENT    0x0020
#define IDA_OLDHDR   0x0040
#define IDA_OLDCALIB 0x0080
#define IDA_POSTHDR  0x0100
#define IDA_RESERVED 0x0200
#define IDA_DMPREC   0x0400
#define IDA_LABEL    0x0800
#define IDA_ARCEOF   0x1000
#define IDA_DASSTAT  0x2000
#define IDA_ISPLOG   0x4000
#define IDA_CORRUPT  0xffff

/*  Digitizer codes  */

#define IDA_DAS     0x01
#define IDA_ARS     0x02
#define IDA_NUMATOD 2

/*  Calibration type codes  */

#define IDA_IMPULSE  1
#define IDA_SQUARE   2
#define IDA_SINE     3
#define IDA_TRIANGLE 4
#define IDA_RANDOM   5

/*  Output devices  */

#define IDA_TAPE  0x01
#define IDA_MODEM 0x02
#define IDA_DTOA  0x04

/*  Calibration channel bitmaps  */

#define IDA_BBCAL  0x01
#define IDA_SPCAL  0x02
#define IDA_INTCAL 0x04

/*  Other misc. constants  */

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef SEEK_SET
#define SEEK_SET 0
#endif

#ifndef SEEK_CUR
#define SEEK_CUR 1
#endif

#ifndef SEEK_END
#define SEEK_END 2
#endif

#ifndef LTL_ENDIAN_ORDER
#define LTL_ENDIAN_ORDER 0x10325476   /* VAX, 80x86   */
#endif

#ifndef BIG_ENDIAN_ORDER
#define BIG_ENDIAN_ORDER 0x76543210   /* Sun, MC680x0 */
#endif

/*  Format codes  */

#define S_UNCOMP    0 /* uncompressed shorts                      */
#define L_UNCOMP    1 /* uncompressed longs                       */
#define IGPP_COMP   2 /* IGPP compression                         */
#define FP32_COMP   3 /* 32 bit fixed point (16:16), uncompressed */
#define FP32_UNCOMP 4 /* 32 bit fixed point (16:16), compressed   */
#define BAD_FORM    5 /* unrecognized format code                 */

/* Mode codes  */

#define CONT       0 /* continuous recording   */
#define TRIG       1 /* triggered              */
#define BAD_MODE   2 /* unrecognized mode code */

/*  Digital filter codes  */
         
#define NONE       0 /* no filtering                               */
#define SDECIM     1 /* straight decimation                        */
#define RDECIM     2 /* decimated with running mean decimator      */
#define BB_5       3 /* broad-band filter,     5 sample/sec output */
#define BB_20      4 /* broad-band filter,    20 sample/sec output */
#define LP         5 /* long-period filter,   10 sec/sample output */
#define MP         6 /* mass-position filter, 10 sec/sample output */
#define DETECT     7 /* detection bandpass filter                  */
#define BB_1       8 /* broad-band filter,     1 sample/sec output */
#define SP_100     9 /* short-period stream, 100 sample/sec output */
#define SP_40     10 /* short-period stream, 100 sample/sec output */
#define BAD_FILT  11 /* unrecognized filter code                   */

/*  Analog input channel codes  */

#define BBZ_HI      0 /* Z broad-band sensor, hi-gain              */
#define BBN_HI      1 /* N broad-band sensor, hi-gain              */
#define BBE_HI      2 /* E broad-band sensor, hi-gain              */
#define BBZ_LO      3 /* Z broad-band sensor, lo-gain              */
#define BBN_LO      4 /* N broad-band sensor, lo-gain              */
#define BBE_LO      5 /* E broad-band sensor, lo-gain              */
#define SPZ_HI      6 /* Z short-period sensor, hi-gain            */
#define SPN_HI      7 /* N short-period sensor, hi-gain            */
#define SPE_HI      8 /* E short-period sensor, hi-gain            */
#define SPZ_LO      9 /* Z short-period sensor, lo-gain            */
#define SPN_LO     10 /* N short-period sensor, lo-gain            */
#define SPE_LO     11 /* E short-period sensor, lo-gain            */
#define MPZ        12 /* Z STS mass position, one gain only        */
#define MPN        13 /* N STS mass position, one gain only        */
#define MPE        14 /* E STS mass position, one gain only        */
#define MODE       15 /* IDA mode (LaCoste), one gain only         */
#define TIDE       16 /* IDA tide (LaCoste), one gain only         */
#define BARO       17 /* microbarograph, one gain only             */
#define ARS_VOLT   18 /* ARS line voltage sensor                   */
#define ARS_TEMP   19 /* ARS temperature sensor                    */
#define DAS_TEMP   20 /* DAS temperature sensor                    */
#define BBZ_24     21 /* single gain 24-bit Z broad-band sensor    */
#define BBN_24     22 /* single gain 24-bit N broad-band sensor    */
#define BBE_24     23 /* single gain 24-bit E broad-band sensor    */
#define UNUSED_CH0 24 /* recognized but normally unused channel    */
#define UNUSED_CH1 25 /* recognized but normally unused channel    */
#define UNUSED_CH2 26 /* recognized but normally unused channel    */
#define UNUSED_CH3 27 /* recognized but normally unused channel    */
#define UNUSED_CH4 28 /* recognized but normally unused channel    */
#define UNUSED_CH5 29 /* recognized but normally unused channel    */
#define UNUSED_CH6 30 /* recognized but normally unused channel    */
#define UNUSED_CH7 31 /* recognized but normally unused channel    */
#define UNUSED_CH8 32 /* recognized but normally unused channel    */
#define UNUSED_CH9 33 /* recognized but normally unused channel    */
#define AUX01      34 /* generic auxillary channel                 */
#define AUX02      35 /* generic auxillary channel                 */
#define AUX03      36 /* generic auxillary channel                 */
#define AUX04      37 /* generic auxillary channel                 */
#define AUX05      38 /* generic auxillary channel                 */
#define AUX06      39 /* generic auxillary channel                 */
#define AUX07      40 /* generic auxillary channel                 */
#define AUX08      41 /* generic auxillary channel                 */
#define AUX09      42 /* generic auxillary channel                 */
#define AUX10      43 /* generic auxillary channel                 */
#define AUX11      44 /* generic auxillary channel                 */
#define AUX12      45 /* generic auxillary channel                 */
#define BAD_CHAN   46 /* invalid channel code                      */

/*  These can be used to dimension arrays  */

#define NFORM (BAD_FORM + 1)
#define NMODE (BAD_MODE + 1)
#define NFILT (BAD_FILT + 1)
#define NCHAN (BAD_CHAN + 1)

/*  Needed by various routines  */

#define BBZ   BAD_CHAN + 1
#define BBN   BAD_CHAN + 2
#define BBE   BAD_CHAN + 3
#define SPZ   BAD_CHAN + 4
#define SPN   BAD_CHAN + 5
#define SPE   BAD_CHAN + 6
#define SMZ   BAD_CHAN + 7
#define SMN   BAD_CHAN + 8
#define SME   BAD_CHAN + 9
#define AE1   BAD_CHAN + 10
#define WK1   BAD_CHAN + 11
#define WK2   BAD_CHAN + 12
#define TLT1  BAD_CHAN + 13
#define TLT2  BAD_CHAN + 14
#define STR1  BAD_CHAN + 15
#define STR2  BAD_CHAN + 16
#define STR3  BAD_CHAN + 17
#define GRV   BAD_CHAN + 18
#define BAR   BAD_CHAN + 19
#define SPZ2  BAD_CHAN + 20
#define SPN2  BAD_CHAN + 21
#define SPE2  BAD_CHAN + 22

/*  OLD Possible time tag error flags  */

#define IDA_BADTQUAL        -12345 /* unrecognized time qual code */
#define TTAG_MIN_EXTTIM     86400L /* 1 day                       */
#define TTAG_MAX_EXTTIM  31708799L /* 367 days - 1 sec            */
#define TTAG_LOW_EXTTIM     0x0001 /* ext time <  TTAG_MIN_EXTTIM */
#define TTAG_BIG_EXTTIM     0x0002 /* ext time >= TTAG_MAX_EXTTIM */
#define TTAG_BAD_EXTTIM     0x0004 /* illegal external time       */
#define TTAG_BAD_OFFSET     0x0008 /* hz time < sys time          */
#define TTAG_BAD_CLKRD      0x0010 /* corrupt time from clock?    */
#define TTAG_BAD_SYSTIM     0x0020 /* beg sys >= end sys time     */

/*  OLD Time structure  */

struct time_tag {
    short year;         /* Data logger's idea of the year           */
    unsigned long mult; /* System and Hz time multiplier (msec/tic) */
    unsigned long sys;  /* System time in clock tics                */
    unsigned long ext;  /* External time in seconds                 */
    unsigned long hz;   /* 1-Hz time in clock tics                  */
    double tru;         /* year + ext + (sys - hz)*mult/1000        */
    char  code;         /* Omega clock quality code                 */
    short qual;         /* Omega code mapped to internal code       */
    int error;          /* error flag (see constants above)         */
};

/* Calibration record */

struct calib_rec {
    int   rev;      /* datalogger format revision code              */
    int   atod;     /* internal digitizer code                      */
    short state;    /* calibration on/off flag                      */
    short func;     /* internal calibration function code           */
    short chan;     /* calibration channel bitmap                   */
    short atten;    /* DAC attenuation factor                       */
    short period;   /* period of calibration function               */
    short width;    /* width of impulse function                    */
    short interval; /* interval between impulses in msecs           */
    short poly;     /* primitive polynomial % 2 for random function */
    short seed;     /* random bit generator seed                    */
    short nchan;    /* number of entries in below array             */
    short gain[IDA_MAXCALCHAN];/* array of channel gains, int. cal */
    struct time_tag ttag;       /* record time tag                  */
};

/* Data record */

struct data_head {
    int   rev;            /* datalogger format revision code          */
    int   atod;           /* internal digitizer code                  */
    int   mode;           /* internal mode code                       */
    int   form;           /* internal format code                     */
    int   chan;           /* internal channel code                    */
    int   filt;           /* internal filter  code                    */
    int   nbytes;         /* number of bytes of raw data              */
    int   srate;          /* digitizer sample rate                    */
    int   decim;          /* filter decimation factor                 */
    int   wrdsiz;         /* uncompressed word size                   */
    int   ffu_shift;      /* Fels' fuck up shift for this stream      */
    int   analog;         /* analog channel code                      */
    unsigned long seqno;  /* packet sequence no (if applicable)       */
    unsigned long order;  /* raw data byte order                      */
    short dl_stream;      /* data logger stream code                  */
    short dl_format;      /* data logger format code                  */
    short dl_channel;     /* data logger channel code                 */
    short dl_filter;      /* data logger filter code                  */
    short nsamp;          /* number of samples                        */
    long  stream;         /* internal stream code (CSS chan)          */
    long  last_word;      /* uncompressed last sample                 */
    float sint;           /* stream sample interval (ie, decim/srate) */
    struct time_tag beg;  /* time tag of first datum                  */
    struct time_tag end;  /* time tag of last  datum                  */
};

struct data_rec {
    struct data_head head;
    char data[IDA_MAXDLEN];
};

/* Configration record */

struct configuration {
    short dl_stream;    /* data logger stream code                  */
    short dl_channel;   /* input channel of stream                  */
    short dl_filter;    /* filter code of filter applied to data    */
    short mode;         /* internal mode code                       */
    short output;       /* bit-map of output devices (internal map) */
    short dl_gain;      /* data logger programable gain setting     */
};

struct detector {
    short chan;    /* bit-map of detector input channels           */
    short key;     /* event detector key channel                   */
    short sta;     /* STA length in samples                        */
    short lta;     /* LTA length in samples                        */
    short thresh;  /* detection STA/LTA ratio                      */
    long  maxlen;  /* maximum trigger length in samples            */
    short voters;  /* minimum number of voters to declare an event */
    short memory;  /* pre-event memory in records                  */
};

struct config_rec {
    int   rev;              /* datalogger format revision code */
    int   atod;             /* internal digitizer code         */
    long  srate;            /* digitizer sample rate           */
    long  nstream;          /* number of streams               */
    short jumpers;          /* backplane jumper settings       */
    struct time_tag ttag;   /* record time tag                 */
    struct detector detect; /* event detector parameters       */
    struct configuration table[IDA_MAXSTREAMS]; /* config. table */
};

/*  Identification record  */

struct ident_rec {
    int   rev;                /* datalogger format revision code */
    int   atod;               /* internal digitizer code         */
    short atod_id;            /* ARS or DAS ident                */
    short atod_rev;           /* ARS or DAS firmware rev         */
    short dsp_rev;            /* DSP firmware rev (DAS only)     */
    short tape_no;            /* tape sequence number            */
    struct time_tag ttag;     /* record time stamp               */
    struct time_tag beg;      /* last tape insert time           */
    struct time_tag end;      /* last tape eject  time           */
    char sname[IDA_SNAMLEN]; /* station code                    */
    int   boot_flag;          /* if set, record was sent on boot */
};

/*  DAS status info  */

struct das_stats {
    long naks;     /* number of naks received (ie, bad packets)  */
    long acks;     /* number of acks recevied (ie, packets sent) */
    long dropped;  /* number of packets dropped by DAS           */
    long timeouts; /* number of xmit timeouts with ARS/ISP       */
    long triggers; /* number of events detected                  */
    int  event;    /* event in progress flag                     */
    int  calib;    /* calibration in progress flag               */
    unsigned long seqno; /* packet sequence number               */
};

/*  Log level codes  */

#define IDA_LOGOPER   1
#define IDA_LOGFATAL  2
#define IDA_LOGERROR  3
#define IDA_LOGWARN   4
#define IDA_LOGINFO   5
#define IDA_LOGDEBUG  6
#define IDA_LOGMAXLEVEL IDA_LOGDEBUG
#define IDA_MAXLOGENTRY 40

/*  A single log entry  */

struct log_entry {
    int   atod;                    /* internal digitizer code  */
    int   level;                   /* log level code           */
    char preamble[IDA_MAXLOGLEN]; /* preamble to message text */
    char message[IDA_MAXLOGLEN];  /* message text             */
};

/*  Log record is an array of log entries  */

struct log_rec {
    int   rev;            /* datalogger format revision code */
    struct time_tag ttag; /* record time tag                 */
    int numentry;         /* number of entries               */
    struct log_entry entry[IDA_MAXLOGENTRY]; /* array of entries  */
};

/* Event record */

struct detection {
    long sta;            /* detector STA at time of event */
    long lta;            /* detector LTA at time of event */
    struct time_tag beg; /* approx. trigger on time       */
    struct time_tag end; /* approx. trigger off time      */
    double duration;     /* approx. duration in seconds   */
};

struct event_rec {
    int   rev;            /* datalogger format revision code */
    struct time_tag ttag; /* record time tag                 */
    int nevent;           /* number of detections            */
    struct detection event[IDA_MAXEVENTS]; /* array of detections  */
};

/* Old-sytle (revs 1-4) configuration table */

struct old_config {
    short nrows;  /* number of rows                  */
    short ncols;  /* number of columns               */
    short table[IDA_MAXOLDCNF]; /* the configration table */
};

/* Old-sytle (revs 1-4) event detector parameters */

struct old_detect {
    short sta;          /* short term average time (msc)           */
    short lta;          /* long term average time (sec)            */
    short thresh_off;   /* threshold offset                        */
    short turnon_fact;  /* turn on:  STA/LTA * 100                 */
    short turnoff_fact; /* turn off: STA/LTA * 100                 */
    short min_rtime;    /* minimum no. seconds to record per event */
    short max_rtime;    /* maximum no. seconds to record per event */
    short min_trgint;   /* minimum no. seconds between events      */
    short dfg;          /* detector filter gain                    */
};

/* Old-style (revs 1-4) header record */

struct old_header {
    int   rev;                 /* datalogger format revision code    */
    char  id;                  /* head record counter                */
    char  sname[7];            /* station name                       */
    char  dl_rev[6];           /* firmware revision code             */
    char  das_id[8];           /* DAS ident                          */
    char  ars_id[8];           /* ARS ident                          */
    short year;                /* data logger's idea of year         */
    unsigned long ext_time;    /* Absolute (external) time tag       */
    struct old_config config;  /* configuration table                */
    short numtap;              /* number of tape drives              */
    short reclen;              /* tape record length                 */
    short rpf;                 /* records per tape file              */
    short fpt;                 /* maximum number of files per tape   */
    struct old_detect bb_trig; /* BB event detector parameters       */
    struct old_detect sp_trig; /* SP event detector parameters       */
    short mt1_errors[3];       /* tape unit 1 error counts           */
    short mt2_errors[3];       /* tape unit 2 error counts           */
    short ars_errors[3];       /* data transmission error counts     */
    short das_errors[4];       /* data acquisition  error counts     */
    short ars_reboots;         /* number of ARS reboots              */
    short das_reboots;         /* number of DAS reboots              */
};

/* Old-style (revs 1-4) calibration record */

struct oldcalib_rec {
    int   rev;  /* datalogger format revision code */
    short unsupported;
};

/* Station location info */

struct ida_coords {
    char  sname[IDA_SNAMLEN]; /* station code              */
    float lat;                /* latitude, decimal degrees */
    float lon;                /* longitude,   "       "    */
    int   elev;               /* surface elevation         */
    int   depth;              /* sensor depth              */
    char  *descrip;           /* site description          */
};

typedef struct ida_coords IDA_COORDS;

/* Sensor orientation info */

struct ida_orient {
    char  sname[IDA_SNAMLEN];   /* station name                  */
    int   chan;                 /* channel code                  */
    long  beg;                  /* begin valid time              */
    long  end;                  /* end     "    "                */
    float hang;                 /* GSE2.0 horizontal orientation */
    float vang;                 /* GSE2.0 vertical orientation   */
    char  instype[IDA_INAMLEN]; /* GSE2.0 instrument name        */
};

typedef struct ida_orient IDA_ORIENT;

/* IDA database object */

struct ida_streammap {
    int filt;       /* data logger filter number  */
    int chan;       /* data logger channel number */
    int trig;       /* triggered flag             */
    int analog;     /* analog channel code        */
    char chname[IDA_CNAMLEN+1]; /* SEED channel name          */
    char *text;     /* free form description      */
    long key;       /* internal stream code       */
};

#define IDA_MAX_CAL_EPOCH 64
#define IDA_MAX_ORIENT    64
#define IDA_MAX_CAL_CHAN  48

struct ida_calinfo {
    char name[IDA_CNAMLEN];
    float calib;
    float calper;
};

struct ida_calepoch {
    long beg;
    long end;
    int nchan;
    struct ida_calinfo chn[IDA_MAX_CAL_CHAN];
};

struct ida_calibmap {
    int nepoch;
    struct ida_calepoch epoch[IDA_MAX_CAL_EPOCH];
};

struct ida_orientmap {
    int nentry;
    struct ida_orient entry[IDA_MAX_ORIENT];
};

struct ida_stainfo {
    char name[IDA_SNAMLEN];
    struct ida_coords    coords;
    struct ida_calibmap  calib;
    struct ida_orientmap orient;
};

struct ida_tqualmap {
    char factory;
    int  internal;
};

struct ida_revmap {
    int code;
    char *text;
};

struct ida_chanmap {
    char name[IDA_MNAMLEN];
    int nstream;
    struct ida_streammap *stream;
};

struct ida_sintmap {
    char sta[IDA_SNAMLEN];
    char chn[IDA_CNAMLEN];
    int sint;
    int datlen;
    int wrdsiz;
};

struct ida_db {
    MUTEX mutex;
    int locked;
    int nmap;
    struct ida_chanmap  *chanmap;
    int nsta;
    struct ida_stainfo  *sta;
    int nqual;
    struct ida_tqualmap *tqual;
    int nrev;
    struct ida_revmap   *rev;
    int nsint;
    struct ida_sintmap  *sint;
    int active_chanmap;
    char *spec;
    DBIO *dbio;
};

typedef struct ida_db IDADB;

/* BEGIN NEW STUFF */

/*  Possible time tag error flags  */

#define IDA_BAD_TQUAL           -12345 /* unrecognized time qual code */
#define IDA_TTAG_MIN_EXTTIM     86400L /* 1 day */
#define IDA_TTAG_MAX_EXTTIM  31708799L /* 367 days - 1 sec */
#define IDA_TTAG_LOW_EXTTIM     0x0001 /* ext time <  IDA_TTAG_MIN_EXTTIM */
#define IDA_TTAG_BIG_EXTTIM     0x0002 /* ext time >= IDA_TTAG_MAX_EXTTIM */
#define IDA_TTAG_BAD_EXTTIM     0x0004 /* illegal external time */
#define IDA_TTAG_BAD_OFFSET     0x0008 /* hz time < sys time */
#define IDA_TTAG_BAD_CLKRD      0x0010 /* corrupt time from clock? */
#define IDA_TTAG_BAD_SYSTIM     0x0020 /* beg sys >= end sys time */

#define IDA_EMBEDDED_YEAR 9999 /* when digitizer has included year in secs field */

/*  Time structure  */

typedef struct {
    UINT32 sys;   /* System time in clock tics */
    UINT32 ext;   /* External time in seconds */
    UINT32 hz;    /* 1-Hz time in clock tics */
    REAL64 epoch; /* year + ext + (sys - hz)*ticrate/1000 */
    char  code;   /* Omega clock quality code */
    int error;    /* error flag (see constants above) */
    BOOL valid;   /* if TRUE, epoch time _might_ be OK */
} IDA_TTAG;

/* Common header */

#define IDA_SOURCE_ARS 3
#define IDA_SOURCE_DAS 2

typedef struct {
    UINT16 year;    /* data logger's notion of the year, often wrong */
    UINT16 useyr;   /* value of the year used to compute epoch time */
    UINT16 ticrate; /* digitizer msec/tic rate */
    IDA_TTAG ttag;  /* packet time stamp */
    UINT16 source;  /* packet source flag */
    UINT32 seqno;   /* sequence number (possibly... ) */
    char site[IDA_SNAMLEN];
} IDA_CMNHDR;

/* Data packet */

#define IDA_TYPE_INT16 1
#define IDA_TYPE_INT32 1

typedef struct {
    IDA_CMNHDR cmn; /* common header */
    struct {
        UINT16 format;  /* format descriptor */
        UINT16 chan;    /* input channel number */
        UINT16 filt;    /* filter code */
        UINT16 nsamp;   /* number of output samples */
        IDA_TTAG tols;  /* time of last sample */
        UINT16 stream;  /* internal stream ident */
    } dl;
    /* The following are derived from the above or are metaknowledge */
    IDA_TTAG tofs;   /* copy of cmn.ttag */
    IDA_TTAG tols;   /* copy of dl.tols */
    double baserate; /* raw digitization rate */
    double sint;     /* actual sample interval */
    int nbytes;      /* number of bytes in data portion */
    int type;        /* data type flag */
    BOOL compressed; /* TRUE if data are compressed */
    BOOL continuous; /* TRUE if data are continuous (non-triggered) */
} IDA_TSHDR;

typedef union {
    INT8 *int8;
    INT16 *int16;
    INT32 *int32;
    UINT8 *uint8;
} IDA_TSDAT;

typedef struct {
    IDA_TSHDR hdr;
    IDA_TSDAT data;
} IDA_TS;

/* Macros */

#ifndef leap_year
#define leap_year(i) ((i % 4 == 0 && i % 100 != 0) || i % 400 == 0)
#endif
#ifndef dysize
#define dysize(i) (365 + leap_year(i))
#endif

#ifdef M_I86 /*  Need to byte-swap on PC's, defined by MS C compiler */
#define SWAB
#endif

#ifdef vax  /*  Need to byte-swap on Vax's  */
#define SWAB
#endif

#ifdef i386  /*  Need to byte-swap on PC's  */
#define SWAB
#endif

#ifdef SWAB
#define SSWAB(a, b) util_sswap((short *) (a), (long) (b))
#define LSWAB(a, b) util_lswap((long *)  (a), (long) (b))
#else
#define SSWAB(a, b)
#define LSWAB(a, b)
#endif /* ifdef SWAB */

#define bad_exttim(tag) (tag->error & TTAG_BAD_EXTTIM)
#define bad_systim(tag) (tag->error & TTAG_BAD_OFFSET)
#define bad_hztim(tag)  (tag->error & TTAG_BAD_OFFSET)
#define bad_offset(tag) (tag->error & TTAG_BAD_OFFSET)

#define valid_rev(rev) (rev_descrip(rev) != NULL)

#define mapped_stream(stream) (stream < 100000)

/* Function Prototypes */

/* atoi.c */
int ida_atoi(char *string);

/* db.c */
IDADB *idadb_init(char *dbdir, int version);
int idadb_supported(IDADB *db, int rev);
int idadb_coords(IDADB *db, char *sname, float *lat, float *lon, int *elev, int *depth, char **descrip);
int idadb_tqual(IDADB *db, char factory, int *internal);
int idadb_sintmap(IDADB *db, struct ida_sintmap *map);
int idadb_calib(IDADB *db, char *sta, char *chan, long tstamp, float *calib, float *calper);
int idadb_inst(IDADB *db, char *sta, char *chn, long tstamp, float *calib, float *calper, float *hang, float *vang, char **inst);
int idadb_defmap(IDADB *db, int rev, int lock);
int idadb_setmap(IDADB *db, char *map, int lock);
long idadb_stream(IDADB *db, struct data_head *head);
int idadb_analog(IDADB *db, long key);
char *idadb_chname(IDADB *db, long key);
char *idadb_seedname(IDADB *db, int chan, int filt, int trig);
int idadb_revchan(IDADB *db, char *cname, short *filt, short *chan);
int idadb_ismapped(IDADB *db, long key);
char *idadb_streamdesc(IDADB *db, long key);
char *idadb_revdesc(IDADB *db, int code);
char *idadb_spec(IDADB *db);
char *idadb_chanmap(IDADB *db);
struct ida_chanmap *idadb_mapptr(IDADB *db);
int idadb_maxstream(IDADB *db);
struct ida_orient *idadb_orient(IDADB *db, char *sname, int analog, double date);
void idadb_print(FILE *fp, IDADB *db);

/* dbcnfrec.c */
int idadb_cnfrec(IDADB *db, struct config_rec *dest, CHAR *src, int rev);

/* dbcnfrec5.c */
int dbcnfrec_rev5(IDADB *db, struct config_rec *dest, CHAR *src);
        
/* dbcrec.c */
int idadb_crec(IDADB *db, struct calib_rec *dest, CHAR *src, int rev);

/* dbcrec5.c */
int dbcrec_rev5(IDADB *db, struct calib_rec *dest, CHAR *src);

/* dbdhead.c */
int idadb_dhead(IDADB *db, struct data_head *dest, CHAR *src, int rev);

/* dbdhead1.c */
int dbdhead_rev1(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead2.c */
int dbdhead_rev2(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead3.c */
int dbdhead_rev3(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead4.c */
int dbdhead_rev4(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead5.c */
int dbdhead_rev5(IDADB *db, struct data_head *dest, CHAR *src);
 
/* dbdhead6.c */
int dbdhead_rev6(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead7.c */
int dbdhead_rev7(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead8.c */
int dbdhead_rev8(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdhead9.c */
int dbdhead_rev9(IDADB *db, struct data_head *dest, CHAR *src);

/* dbdrec.c */
int idadb_drec(IDADB *db, struct data_rec *dest, CHAR *src, int rev, int shift, int swab);

/* dbdrec1.c */
int dbdrec_rev1(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec2.c */
int dbdrec_rev2(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec3.c */
int dbdrec_rev3(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec4.c */
int dbdrec_rev4(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec5.c */
int dbdrec_rev5(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec6.c */
int dbdrec_rev6(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec7.c */
int dbdrec_rev7(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec8.c */
int dbdrec_rev8(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbdrec9.c */
int dbdrec_rev9(IDADB *db, struct data_rec *dest, CHAR *src, int swab);

/* dbevtrec.c */
int idadb_evtrec(IDADB *db, struct event_rec *dest, CHAR *src, int rev);

/* dbevtrec5.c */
int dbevtrec_rev5(IDADB *db, struct event_rec *dest, CHAR *src);

/* dbfilltime.c */
struct time_tag *dbfilltime(IDADB *db, struct time_tag *tag);

/* dbidrec.c */
int idadb_idrec(IDADB *db, struct ident_rec *dest, CHAR *src, int rev);

/* dbidrec5.c */
int dbidrec_rev5(IDADB *db, struct ident_rec *dest, CHAR *src);

/* dblogrec.c */
int idadb_logrec(IDADB *db, struct log_rec *dest, CHAR *src, int rev);

/* dblogrec5.c */
int dblogrec_rev5(IDADB *db, struct log_rec *dest, CHAR *src);

/* dcmp.c */
long ida_dcmp(long *dest, CHAR *src, int nsamp);

/* dhlen.c */
void ida_dhlen(int rev, long stream, int *head, int *data);

/* dhstr.c */
char *idadb_dhstr(IDADB *db, struct data_head *head, char *buffer);

/* discard.c */
long read_discards(char *fname);
int discard(long num);
long get_discards(long **ptr);
void free_discards(VOID);

/* ext1hzoff.c */
double ida_ext1hzoff(struct time_tag *tag);

/* hrec.c */
int ida_hrec(struct old_header *dest, CHAR *src, int rev);

/* hrec04.c */
int hrec0_rev4(struct old_header *dest, CHAR *src);

/* hrec1.c */
int hrec_rev1(struct old_header *dest, CHAR *src);

/* hrec14.c */
int hrec1_rev4(struct old_header *dest, CHAR *src);

/* hrec3.c */
int hrec_rev3(struct old_header *dest, CHAR *src);

/* loadsta.c */
void ida_loadsta(char *sta, char *buffer, int rev);

/* loadyr.c */
void ida_loadyr(int year, char *buffer, int rev);

/* orient.c */
int ida_numorient(VOID);
struct ida_orient *ida_allorient(VOID);
struct ida_orient *ida_orient(char *sname, int analog, double date);

/* rtype.c */
int ida_rtype(char *buf, int rev);

/* sampok.c */
int ida_sampok(struct data_head *dhead);

/* select.c */
int keep_stream_init(VOID);
int keep_stream(long stream);
int streamselect_update(char *string);

/* srec.c */
int ida_srec(struct das_stats *dest, CHAR *src, int rev);

/* srec8.c */
int srec_rev8(struct das_stats *dest, CHAR *src);

/* timerr.c */
int ida_timerr(struct data_head *crnt, struct data_head *prev, int *sign, unsigned long *errptr);
unsigned long ida_offerr(struct data_head *dhead, int *sign, unsigned long *expected);

/* timstr.c */
char *sys_timstr(unsigned long tics, unsigned long mult);
char *ext_timstr(unsigned long sec);
char *tru_timstr(struct time_tag *tag);

/* unpack.c */
int idaUnpackD(UINT8 *start, IDA_TS *out, int rev);

/* version.c */
char *idaVersionString(VOID);
VERSION *idaVersion(VOID);

#ifdef __cplusplus
}
#endif

#endif

/* Revision History
 *
 * $Log: ida.h,v $
 * Revision 1.10  2004/06/25 18:34:56  dechavez
 * C++ compatibility
 *
 * Revision 1.9  2004/06/24 16:56:27  dechavez
 * removed unecessary system includes
 *
 * Revision 1.8  2004/06/21 19:37:40  dechavez
 * changed time_tag "true" field to "tru"
 *
 * Revision 1.7  2004/04/23 00:37:19  dechavez
 * dbio support
 *
 * Revision 1.6  2003/10/16 17:50:03  dechavez
 * added some new general datatypes for future use, updated prototypes
 *
 * Revision 1.5  2002/02/21 22:32:54  dec
 * increase max channel name length to 8 chars
 *
 * Revision 1.4  2001/05/20 15:56:01  dec
 * switch to platform.h MUTEX
 *
 * Revision 1.3  2001/05/07 22:28:11  dec
 * added platform.h, removed CHAR references
 *
 * Revision 1.2  2000/03/16 06:13:14  dec
 * Added sintmap to idadb
 *
 * Revision 1.1.1.1  2000/02/08 20:20:22  dec
 * import existing IDA/NRTS sources
 *
 */
