#pragma ident "$Id: pack.c,v 1.3 2003/11/19 23:49:14 dechavez Exp $"
/*======================================================================
 *
 *  Pack/Unpack ISI data objects into/out of NBO
 *
 *====================================================================*/
#include "isi.h"
#include "util.h"

/* Pack/unpack coordinates */

int isiPackCoords(UINT8 *start, ISI_COORDS *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilPackREAL32(ptr, src->lat);
    ptr += utilPackREAL32(ptr, src->lon);
    ptr += utilPackREAL32(ptr, src->elev);
    ptr += utilPackREAL32(ptr, src->depth);

    return (int) (ptr - start);
}

int isiUnpackCoords(UINT8 *start, ISI_COORDS *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilUnpackREAL32(ptr, &dest->lat);
    ptr += utilUnpackREAL32(ptr, &dest->lon);
    ptr += utilUnpackREAL32(ptr, &dest->elev);
    ptr += utilUnpackREAL32(ptr, &dest->depth);

    return (int) (ptr - start);
}

/* Pack/unpack instrument data */

int isiPackInst(UINT8 *start, ISI_INST *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilPackREAL32(ptr, src->calib);
    ptr += utilPackREAL32(ptr, src->calper);
    ptr += utilPackREAL32(ptr, src->hang);
    ptr += utilPackREAL32(ptr, src->vang);
    ptr += utilPackBytes(ptr, (UINT8 *) src->type, ISI_INAMLEN);

    return (int) (ptr - start);
}

int isiUnpackInst(UINT8 *start, ISI_INST *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilUnpackREAL32(ptr, &dest->calib);
    ptr += utilUnpackREAL32(ptr, &dest->calper);
    ptr += utilUnpackREAL32(ptr, &dest->hang);
    ptr += utilUnpackREAL32(ptr, &dest->vang);
    ptr += utilUnpackBytes(ptr, (UINT8 *) dest->type, ISI_INAMLEN);
    dest->type[ISI_INAMLEN] = 0;

    return (int) (ptr - start);
}

/* Pack/unpack a datum description */

int isiPackDatumDescription(UINT8 *start, ISI_DATA_DESC *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilPackBytes(ptr, &src->comp, 1);
    ptr += utilPackBytes(ptr, &src->type, 1);
    ptr += utilPackBytes(ptr, &src->order, 1);
    ptr += utilPackBytes(ptr, &src->size, 1);

    return (int) (ptr - start);
}

int isiUnpackDatumDescription(UINT8 *start, ISI_DATA_DESC *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilUnpackBytes(ptr, &dest->comp, 1);
    ptr += utilUnpackBytes(ptr, &dest->type, 1);
    ptr += utilUnpackBytes(ptr, &dest->order, 1);
    ptr += utilUnpackBytes(ptr, &dest->size, 1);

    return (int) (ptr - start);
}

/* Pack/unpack a sample rate */

int isiPackSrate(UINT8 *start, ISI_SRATE *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilPackINT16(ptr, src->factor);
    ptr += utilPackINT16(ptr, src->multiplier);

    return (int) (ptr - start);
}

int isiUnpackSrate(UINT8 *start, ISI_SRATE *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilUnpackINT16(ptr, &dest->factor);
    ptr += utilUnpackINT16(ptr, &dest->multiplier);

    return (int) (ptr - start);
}

/* Pack/unpack a time stamp */

int isiPackTstamp(UINT8 *start, ISI_TSTAMP *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilPackREAL64(ptr, src->value);
    ptr += utilPackUINT16(ptr, src->status);

    return (int) (ptr - start);
}

int isiUnpackTstamp(UINT8 *start, ISI_TSTAMP *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilUnpackREAL64(ptr, &dest->value);
    ptr += utilUnpackUINT16(ptr, &dest->status);

    return (int) (ptr - start);
}

/* Pack/unpack a stream name */

int isiPackStreamName(UINT8 *start, ISI_STREAM_NAME *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilPackBytes(ptr, (UINT8 *) src->sta, ISI_STALEN);
    ptr += utilPackBytes(ptr, (UINT8 *) src->chn, ISI_CHNLEN);
    ptr += utilPackBytes(ptr, (UINT8 *) src->loc, ISI_LOCLEN);

    return (int) (ptr - start);
}

int isiUnpackStreamName(UINT8 *start, ISI_STREAM_NAME *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += utilUnpackBytes(ptr, (UINT8 *) dest->sta, ISI_STALEN);
    dest->sta[ISI_STALEN] = 0;
    ptr += utilUnpackBytes(ptr, (UINT8 *) dest->chn, ISI_CHNLEN);
    dest->chn[ISI_CHNLEN] = 0;
    ptr += utilUnpackBytes(ptr, (UINT8 *) dest->loc, ISI_LOCLEN);
    dest->loc[ISI_LOCLEN] = 0;

    return (int) (ptr - start);
}

/* Pack/unpack a data request */

int isiPackStreamRequest(UINT8 *start, ISI_STREAM_REQUEST *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += isiPackStreamName(ptr, &src->name);
    ptr += utilPackREAL64(ptr, src->beg);
    ptr += utilPackREAL64(ptr, src->end);

    return (int) (ptr - start);
}

int isiUnpackStreamRequest(UINT8 *start, ISI_STREAM_REQUEST *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += isiUnpackStreamName(ptr, &src->name);
    ptr += utilUnpackREAL64(ptr, &src->beg);
    ptr += utilUnpackREAL64(ptr, &src->end);

    return (int) (ptr - start);
}

/* Pack/unpack a configuration packet */

int isiPackStreamCnf(UINT8 *start, ISI_STREAM_CNF *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += isiPackStreamName(ptr, &src->name);
    ptr += isiPackSrate(ptr, &src->srate);
    ptr += isiPackCoords(ptr, &src->coords);
    ptr += isiPackInst(ptr, &src->inst);

    return (int) (ptr - start);
}

int isiUnpackStreamCnf(UINT8 *start, ISI_STREAM_CNF *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += isiUnpackStreamName(ptr, &dest->name);
    ptr += isiUnpackSrate(ptr, &dest->srate);
    ptr += isiUnpackCoords(ptr, &dest->coords);
    ptr += isiUnpackInst(ptr, &dest->inst);

    return (int) (ptr - start);
}

/* Pack/unpack a state of health packets */

int isiPackStreamSoh(UINT8 *start, ISI_STREAM_SOH *src)
{
UINT8 *ptr;

    ptr = start;

    ptr += isiPackStreamName(ptr, &src->name);
    ptr += isiPackTstamp(ptr, &src->tofs);
    ptr += isiPackTstamp(ptr, &src->tols);
    ptr += utilPackREAL64(ptr, src->tslw);
    ptr += utilPackUINT32(ptr, src->nseg);
    ptr += utilPackUINT32(ptr, src->nrec);

    return (int) (ptr - start);
}

int isiUnpackStreamSoh(UINT8 *start, ISI_STREAM_SOH *dest)
{
UINT8 *ptr;

    ptr = start;

    ptr += isiUnpackStreamName(ptr, &dest->name);
    ptr += isiUnpackTstamp(ptr, &dest->tofs);
    ptr += isiUnpackTstamp(ptr, &dest->tols);
    ptr += utilUnpackREAL64(ptr, &dest->tslw);
    ptr += utilUnpackUINT32(ptr, &dest->nseg);
    ptr += utilUnpackUINT32(ptr, &dest->nrec);

    return (int) (ptr - start);
}

/* Pack/unpack a CD status structure */

int isiPackCdstatus(UINT8 *start, ISI_CD_STATUS *src)
{
UINT8 *ptr;

    ptr = start;
    *ptr = src->data;     ptr += 1;
    *ptr = src->security; ptr += 1;
    *ptr = src->misc;     ptr += 1;
    *ptr = src->voltage;  ptr += 1;
    ptr += utilPackREAL64(ptr, src->sync);
    ptr += utilPackUINT32(ptr, src->diff);

    return (int) (ptr - start);
}

int isiUnpackCdstatus(UINT8 *start, ISI_CD_STATUS *dest)
{
UINT8 *ptr;

    ptr = start;
    dest->data     = *ptr; ptr += 1;
    dest->security = *ptr; ptr += 1;
    dest->misc     = *ptr; ptr += 1;
    dest->voltage  = *ptr; ptr += 1;
    ptr += utilUnpackREAL64(ptr, &dest->sync);
    ptr += utilUnpackUINT32(ptr, &dest->diff);

    return (int) (ptr - start);
}

/* Pack/unpack a generic time series packet */

int isiPackGenericTSHDR(UINT8 *start, ISI_GENERIC_TSHDR *src)
{
UINT8 *ptr;
static char *fid = "isiPackGenericTSHDR";

    ptr = start;

    ptr += isiPackStreamName(ptr, &src->name);
    ptr += isiPackSrate(ptr, &src->srate);
    ptr += isiPackTstamp(ptr, &src->tofs);
    ptr += isiPackTstamp(ptr, &src->tols);
    ptr += isiPackCdstatus(ptr, &src->status);
    ptr += utilPackUINT32(ptr, src->nsamp);
    ptr += utilPackUINT32(ptr, src->nbytes);
    ptr += isiPackDatumDescription(ptr, &src->desc);

    return (int) (ptr - start);
}

int isiUnpackGenericTSHDR(UINT8 *start, ISI_GENERIC_TSHDR *dest)
{
UINT8 *ptr;

    ptr = start;
    ptr += isiUnpackStreamName(ptr, &dest->name);
    ptr += isiUnpackSrate(ptr, &dest->srate);
    ptr += isiUnpackTstamp(ptr, &dest->tofs);
    ptr += isiUnpackTstamp(ptr, &dest->tols);
    ptr += isiUnpackCdstatus(ptr, &dest->status);
    ptr += utilUnpackUINT32(ptr, &dest->nsamp);
    ptr += utilUnpackUINT32(ptr, &dest->nbytes);
    ptr += isiUnpackDatumDescription(ptr, &dest->desc);

    return (int) (ptr - start);
}

/* Unpack a generic TS packet, keeping data in its place */

int isiUnpackGenericTS(UINT8 *start, ISI_GENERIC_TS *dest)
{
UINT8 *ptr;

    ptr = start;
    ptr += isiUnpackGenericTSHDR(ptr, &dest->hdr);
    dest->data = (VOID *) ptr;
    ptr += dest->hdr.nbytes;

    return (int) (ptr - start);
}

/* Revision History
 *
 * $Log: pack.c,v $
 * Revision 1.3  2003/11/19 23:49:14  dechavez
 * cast utilPackBytes/utilUnpackBytes args to calm compiler
 *
 * Revision 1.2  2003/11/03 23:18:50  dechavez
 * added isiUnpackGenericTS()
 *
 * Revision 1.1  2003/10/16 15:38:52  dechavez
 * Initial release
 *
 */
