
/* file PFUSRDEF.H
 *      ==========
 *
 * version 5, 13-Nov-2004
 *
 * prototypes of module POLFILTR.C
 * K. Stammler, 7-APR-91
 */


#ifndef __PFUSRDEF
#define __PFUSRDEF


/*-----------------------------------------------------------------------*/


void pf_filter( REAL xi[], REAL yi[], REAL zi[], REAL xo[],
	REAL yo[], REAL zo[], long lth, int n, REAL p, int cmreset );

/* filters 3-dim seismogram (x,y,z)
 *
 * parameters of routine
 * REAL       xi[], yi[], zi[];   input; 3-dim input seismogram
 * REAL       xo[], yo[], zo[];   output; filtered 3-dim seismogram
 * long       lth;                input; length of traces
 * int        n;                  input; coherence length
 * REAL       p;                  input; exponent of filter factor
 * int        cmreset;            input; the covariance matrix is
 *                                       recomputed after cmreset steps
 */


/*-----------------------------------------------------------------------*/


void pf_vecfilter( REAL xi[], REAL yi[], REAL zi[], REAL xo[],
	REAL yo[], REAL zo[], long lth, int n, REAL p, int cmreset, STATUS *status );

/* filters 3-dim seismogram (x,y,z)
 *
 * parameters of routine
 * REAL       xi[], yi[], zi[];   input; 3-dim input seismogram
 * REAL       xo[], yo[], zo[];   output; filtered 3-dim seismogram
 * long       lth;                input; length of traces
 * int        n;                  input; coherence length
 * REAL       p;                  input; exponent of filter factor
 * int        cmreset;            input; the covariance matrix is
 *                                       recomputed after cmreset steps
 * STATUS     *status;            output; return status
 */


/*-----------------------------------------------------------------------*/


void pf_eigenvector( REAL xi[], REAL yi[], REAL zi[], REAL xo[],
	REAL yo[], REAL zo[], long lth, int n, REAL p, int cmreset, STATUS *status );

/* Returns normalized and weighted largest eigenvector for each sample position
 *
 * parameters of routine
 * REAL       xi[], yi[], zi[];   input; 3-dim input seismogram
 * REAL       xo[], yo[], zo[];   output; largest eigenvector
 * long       lth;                input; length of traces
 * int        n;                  input; coherence length
 * REAL       p;                  input; exponent of filter factor
 * int        cmreset;            input; the covariance matrix is
 *                                       recomputed after cmreset steps
 * STATUS     *status;            output; return status
 */


/*-----------------------------------------------------------------------*/


void pf_poldir( REAL xi[], REAL yi[], REAL zi[], REAL xo[],
	REAL yo[], REAL zo[], long lth, SAMPLE poldir[], REAL p );

/* computes scalar product of each data point of the 3-dim input trace
 * and a given vector poldir (both vectors are normalized before).  The
 * data point then is weighted with the result powered by 'p'.
 *
 * parameters of routine
 * REAL       xi[], yi[], zi[];   input; 3-dim input seismogram
 * REAL       xo[], yo[], zo[];   output; filtered 3-dim seismogram
 * long       lth;                input; length of traces
 * int        poldir[];           input; polarization direction
 * REAL       p;                  input; exponent of filter factor
 */


/*-----------------------------------------------------------------------*/


void pf_bazdir( REAL xi[], REAL yi[], REAL zi[], REAL xo[],
	REAL yo[], REAL zo[], long lth, REAL baz, REAL p );

/* computes scalar product of each data point of the N,E input trace
 * and a given back-azimuth (both vectors are normalized before).  The
 * 3-dim data point then is weighted with the result powered by 'p'.
 *
 * parameters of routine
 * REAL       xi[], yi[], zi[];   input; 3-dim input seismogram
 * REAL       xo[], yo[], zo[];   output; filtered 3-dim seismogram
 * long       lth;                input; length of traces
 * REAL       baz;                input; back-azimuth in deg
 * REAL       p;                  input; exponent of filter factor
 */


/*-----------------------------------------------------------------------*/

#endif /* __PFUSRDEF */
