#!/usr/bin/env python

import sys, time, os, signal, seis
from bisect import bisect, insort
from seiscomp.detector import Trigger as Pick, TriggerInput as PickInput
from seis import Time, Event, delta, ttt
from Numeric import arange, average, concatenate, sin, cos, exp, pi, sqrt, zeros, array, arctan, arcsin, arctan2, sum
from seiscomp.autoloc_locsat    import LocSAT as Locate, PhaseArrival, Station
from seiscomp.autoloc_beam      import Beam

from seiscomp.autoloc_location  import print_origin_procalert, print_summary_line, LocationFailed, remove_redundant_trial_origins, sort_trial_origins, origin_update, good_origin, compare_origin_quality, dist_wt, print_origin_seismic_handler, print_origin_sortlist

from seiscomp.autoloc_location  import compute_magnitude_mb,compute_magnitude_ml
from seiscomp.autoloc_misc      import sorted_by_time, pol_weight, \
                    azi_inc_to_vector, time_average, list_contains_list, \
                    list_overlap, average_origin
from seiscomp.event_db import Origin, Association, StationMagnitude
import seiscomp.autoloc_location
from seis.msg import Logger

max_rms_res = 3.5
max_con_reg = 300

def loc2org(loc):
    # XXX This is transient code that will go away after
    # XXX completing the migration Location -> Origin
    org = Origin(id=loc.id, time=loc.time,
                 latitude=loc.lat, longitude=loc.lon, depth=loc.dep,
                 time_error = loc.time_err, latitude_error = loc.lat_err,
                 longitude_error = loc.lon_err, depth_error = loc.dep_err) 
#   org.error = OriginError(id=loc.id,
#                   time_error = loc.time_err, latitude_error = loc.lat_err,
#                   longitude_error = loc.lon_err, depth_error = loc.dep_err)
    return org

def org2loc(org):
    org2      = seiscomp.autoloc_location.Origin()
    org2.id   = org.id
    org2.time = org.time
    org2.lat  = org.latitude
    org2.lon  = org.longitude
    org2.dep  = org.depth

    org2.time_err = org.time_error
    org2.lat_err  = org.latitude_error
    org2.lon_err  = org.longitude_error
    org2.dep_err  = org.depth_error

#   if org.error is not None:
#       org2.time_err = org.error.time_error
#       org2.lat_err  = org.error.latitude_error
#       org2.lon_err  = org.error.longitude_error
#       org2.dep_err  = org.error.depth_error
#   else:
#       org2.time_err = org2.lat_err = org2.lon_err = org2.dep_err = 0.
        
    return org2

class AssociationList(list):

    def find_by_origin_id(self, origin_id):
        pass

    def find_by_pick_id(self, pick_id):
        pass

    def find_by(self, origin_id=None, pick_id=None, used=None):
        result = AssociationList()
        for a in self:
            if origin_id is not None and a.origin_id != origin_id:
                continue
            if pick_id is not None and a.pick_id != pick_id:
                continue
            if used is not None and a.used != used:
                continue

            result.append(a)

        return result


def read_station_inf(station_inf):

    from seiscomp.autoloc_sta \
        import read_station_inf as XXX_OLD_read_station_inf

    try:
        stations = {}
        for sta_inf in XXX_OLD_read_station_inf(station_inf).values():
            net, sta, lat, lon, alt, weight = sta_inf
            if weight < 0.5:
                continue
            stations[sta] = Station(net=net,code=sta,lat=lat,lon=lon, alt=alt)
    except:
        raise IOError, "error reading station file %s" % station_inf
    return stations


def picks_to_arrivals(picks):

    # Convert a list of picks to a list of arrivals, sorted by arrival
    # time, suitable to be pased to LocSAT. 

    arrivals = []

    for p in picks:
        a = PhaseArrival()
        a.id   = p.id
        a.pha  = "P"
        a.time = p.on
        a.net  = p.net
        a.sta  = p.sta
        arrivals.append( (a.time, a) )

#   arrivals.sort()
    arrivals = [ a for t,a in arrivals ]
    return arrivals


def remove_associated_picks(trial_origins, associated):
    new_trial_origins = []

    for i, org in enumerate(trial_origins): # XXX
        org.trgs = [ pck for pck in org.trgs if pck.id not in associated ]
        new_trial_origins.append(org)

    return new_trial_origins


def remove_outlying_picks(trial_origins):
    # takes care of the situation where one station is much farther away from
    # the hypothetical hypocenter than the others.
    global stations
    new_trial_origins = []
    for org in trial_origins: # XXX
        if len(org.trgs)<5:
            continue
        dist_pck = []
        sta_list = []
        for pck in org.trgs:
            # don't allow two picks of same station in a group
            # XXX this doesn't actually belong here
            if pck.sta in sta_list:
                continue
            d, az, baz = delta.delazi2(org, stations[pck.sta])
            dist_pck.append( (d,pck) ) 
            sta_list.append(pck.sta)
        # XXX THIS IS A BRUTAL HACK XXX
        dist_pck.sort()
        try:
            if   dist_pck[-1][0] > 60 and dist_pck[-2][0] < 20:
                org.remove_arrival(dist_pck[-1][1].id)
            elif dist_pck[-2][0] > 60 and dist_pck[-3][0] < 20:
                org.remove_arrival(dist_pck[-2][1].id)
                org.remove_arrival(dist_pck[-1][1].id)
#           if   dist_pck[-1][0] > 3*dist_pck[-2][0]:
#               dist_pck = dist_pck[:-1]
#           elif dist_pck[-2][0] > 3*dist_pck[-3][0]:
#               dist_pck = dist_pck[:-2]
        except:
            continue
   
        org.trgs = [ pck for d,pck in dist_pck ]
        if len(org.trgs) > 4: # XXX
            new_trial_origins.append(org)
    
    return new_trial_origins


class AutoLoc(object):

    dtmax = 60.
    nmin  = 6
    maxage = 24*3600

    def __init__(self, beams, stations, config, logger, db = None):

        self.verbosity = config.verbosity
        self.tmpdir    = config.tmpdir
        self.realtime  = config.realtime
        self.logger    = logger
        self.db        = db

        if self.verbosity>=1:
            self.logger.write("Initializing AutoLoc\n")

        if self.verbosity >= 3:
                verbose = True
        else:   verbose = False
        self.locate = Locate(stations, tmpdir=self.tmpdir,
                             logger=self.logger, verbose=verbose)
        self.__setup_stations(stations)
        self.min_time = seis.Time(time1970=0.)

        if self.db:
            self.__init_from_db()
        else:
            self.origins = []
            self.origin_ids = {}
            self.all_picks = {}
            self.associated = {} # associated pick ID's/events

        self.__setup_beams(beams)
            
    def __init_from_db(self):

        now = seis.time()
        twin = (now-2*self.maxage, now)
        twin = None # XXX XXX XXX

        self.all_picks = {}
        self.associated   = {} # associated pick ID's/events

        self.origins = []
        self.origin_ids = {}
        for org in self.db.get_origins(twin=twin):
            org2 = org2loc(org)
            self.add_origin(org2)
            assos = org.get_associations()
            picks = []
            for asso in assos:
                pick = self.db.pick(asso.pick_id)
                pick.on = pick.time # XXX
                picks.append(pick)
                pick.phase = asso.phase # XXX transient
            for pick in picks:
                self.associated[pick.id] = org2
                self.all_picks[pick.id] = pick
            org2.trgs = picks[:]
            org2.arr  = picks_to_arrivals(picks[:])
            for arr in org2.arr:
                for asso in assos:
                    if asso.pick_id==arr.id:
                        arr.res  = asso.residual
                        arr.used = 0
                        if asso.used: arr.used = 1
                sta = self.get_station(arr.sta)
                arr.dist, arr.azi, baz = delta.delazi2(org2, sta)
            if len(org2.arr) > 0:
                  org2.rms_res = sqrt(average(array([a.res*dist_wt(a.dist)
                                                for a in org2.arr])**2))
            else: org2.rms_res = 0.
            # self.all_picks.extend(picks)
            # self.associations.extend(assos)

            if self.verbosity>=3:
                msg = print_origin_procalert(org2)
                self.logger.write("%s\n" % msg)


    def __del__(self):
        if self.verbosity>=1:
            self.logger.write("Shutting down AutoLoc...\n")
        try: del self.locate
        except: pass
        if self.verbosity>=1:
            self.logger.write("AutoLoc shutdown completed.\n")
    
    def __setup_stations(self, stations):
        self.stations = [(s.code, s) for s in stations.values()]
        self.stations.sort()
        self.stations = [s      for c,s in self.stations]

        # XXX make it a dict of indices
        self.staindex = {}
        for i,s in enumerate(self.stations):
            self.staindex[s.code] = i

    def __setup_beams(self, beams):
        
        self.beams = [ Beam(lat=lat, lon=lon, dep=dep, diam=diam,
                            maxdist=maxdist, nmin=nmin) 
                       for lat,lon,dep,diam,maxdist,nmin in beams ]

        # compute P arrivals for each station used for this beam
        for ib, beam in enumerate(self.beams):

            if self.verbosity >= 2:
                sys.stderr.write("setting up beam #%5d\r" % ib)

            beam.setup(self.stations, self.staindex)

        if self.verbosity >= 2:
            sys.stderr.write("                      \r")


    def __current_time(self):
        # determine the (real or approximate) current time from the picks.
        now = seis.time()

        if not self.realtime:
            picks = [ (pck.on, pck) for pck in self.all_picks.values() ]
            picks.sort()
            # The pick list is sorted and we use the pick time of the
            # 5th most recent station, to avoid times in the future due to
            # GPS or other timing errors
            stations = []
            for t, pck in picks[::-1]:
                if pck.sta in stations:
                    continue
                stations.append(pck.sta)
                now = t 
                if len(stations) == 5:
                    break
        return now


    def __update_current_time(self, pck):
        # to replace __current_time
        pass


    def __read_station_info(self):
        # XXX yet to be implented
        pass


    def add_origin(self, org):
        # add an origin to the internal list of origins
        self.origins.append(org)
        self.origins = sorted_by_time(self.origins)
        while org.id in self.origin_ids:
            if self.verbosity>=1:
                self.logger.write("duplicate origin id %s\n" % org.id)
            org.id = org.id+"X"
        self.origin_ids[org.id] = org


    def truncate(self):
        # truncate list of arrivals, i.e. remove old arrivals
        now = self.__current_time()
        self.min_time = now - self.maxage

        todelete = [ pck.id for pck in self.all_picks.values()
                            if  pck.on < self.min_time ]

        for pck_id in todelete:
            del self.all_picks[pck_id]

        for beam in self.beams:
            beam.truncate(self.min_time)

        self.origins = [ org for org in self.origins \
                           if now - org.time <= self.maxage]
        # XXX XXX truncate associated XXX XXX


    def get_station(self, code):
        # retrieves station information from the internal station database
        return self.stations[self.staindex[code]]


    def __pick_matches_origin(self, org, pck):
        # test if a pick is compatible with any of
        # the origins that are already in the list
        try:
                sta = self.get_station(pck.sta)
        except KeyError:
                # XXX Warning, that station could not be found in DB
                # or maybe check that in feed()
                return False

        # This has happened at least once - prevent it as
        # such an "origin" doesn't make sense anyway
        if not 0 <= org.dep <= 800: return False

        d,a,b = delta.delazi2(org, sta)
        arrivals = ttt.compute(d, org.dep)
        if   d< 30: phases = [ "Pn", "Pb", "Pg", "P" ]
        elif d<120: phases = [ "P", "Pdiff" ]
        else:       phases = [ "PKiKP", "PKPdf" ]
        arr = arrivals.findFirstOf(phases)
        if arr:
            dt = pck.on - arr.time - org.time
            if abs(dt)<100:
                dt_max = 10./dist_wt(d)
                if self.verbosity>=2:
                    self.logger.write("__pick_matches_origin %s %s dt =%.2f " \
                              "dt_max=%.2f\n" % (pck.id, org.id, dt, dt_max))
                if abs(dt) < dt_max:
                    return True

        # test for phases, which we don't currently take into account
        # XXX it may of course be possible that we compare a secondary phase
        # with a wrongly located event. It would be better to repeat this
        # whenever a origin gets improved. No high priority, though, because
        # the big events that my generate e.g. a large PP will be located well
        # enough by the time PP arrives.
        arrivals.filter(["pP","PP"])
        for arr in arrivals:
            # don't try to associate depth phases for known shallow events
            if arr.phase[0] in "ps" and org.dep < 150:
                continue
            if abs(org.time + arr.time - pck.on) < 12:
                pck.ignore = True
                if self.verbosity>=1:
                    self.logger.write("associated %s with %s phase of " \
                        "event %s - ignored\n" % (pck.id, arr.phase, org.id))
                return False

        return False


    def compare_with_known_events(self, pck):
        # tries to match new pick against existing events

        pck.ignore = False

        # iterate over known origins starting with the most recent
        for org in self.origins[::-1]:

#           if not good_origin(org):   # XXX This must be configurable
#               continue

            if pck.on < org.time:
                continue

            if pck.on - 1800 > org.time:
                break

            if self.__pick_matches_origin(org, pck):
                return org


    def perhaps_improve_origin(self, org, pck):
        # adds the pick 'pck' to the existing origin 'org'
        # and runs the location program. If the origin can be improved, the
        # pick is added to it

        pcks = org.trgs + [ pck ]
        old_rms_res = org.rms_res
        start = (org.time, org.lat, org.lon, org.dep)
        tmp_org, arr = self.locate(pcks, start, autorefine=3)
        tmp_org.trgs = pcks
        tmp_org.arr  = arr
#       tmp_org.rms_res = sqrt(average(array([a.res for a in arr])**2))

        comp = compare_origin_quality(org, tmp_org, self.logger)
        msg = "compare_origin_quality X %s = %.4f" % (org.id, comp)
#       if tmp_org.rms_res > max_rms_res or tmp_org.rms_res > 1.5*old_rms_res:
        if tmp_org.rms_res > max_rms_res or comp < 0.:
                    raise LocationFailed, "Location failed"
        msg += "\nUPDATING %s   RMS= %.2f  %.2f\n" % \
                            (str(org.id), org.rms_res, tmp_org.rms_res)
        if self.verbosity>=1:
            self.logger.write("%s\n" % msg)
        origin_update(org, tmp_org)


    def __feed(self, pck0):

        # If the pick is already present in list of all picks...
        if pck0.id in self.all_picks:
            # ... only update attributes
            pck = self.all_picks[pck0.id]   # find original pick
            pck.update(pck0)                # update attributes

            # However, the updated attributes need to require an update of
            # origin/magnitudes as well. Therefore, if the pick has
            # already been associated to an event, this event is simply
            # returned in order to make the calling process update it.
            if pck0.id in self.associated: # XXX ist das so richtig?
                org = self.associated[pck0.id]
                # updated an already associated pick, without need to
                # recompute the origin
                return 1, [ org ]

            return 0, [] # updated an unassociated pick

        # if the pick has already expired
        if pck0.on < self.min_time:
            if self.verbosity>=1:
                self.logger.write("ignoring expired pick %s\n" % pck0.id)
            return 0, []

        # see if there were many picks in the last 'look_back' seconds
        # if that it the case, the minimum pick amplitude must be larger
        ignore_new_trigger = False
        look_back = 900
        pcnt = 0
        for p in self.all_picks.values():
            if pck0.sta != p.sta: continue
            if pck0.net != p.net: continue
            if pck0.time - p.time > look_back: continue
            pcnt += 2
        # The pick is only considered if it's amplitude exceeds the number of
        # picks seen from that station during the last half hour.
        if pcnt > pck0.ampl_max: # XXX This is a untested criterion
            ignore_new_trigger = True

        # XXX vvv This is ugly!
        pck0.pha, pck0.time = "P", pck0.on
        pck0.used = True # XXX
        pck0.ignore = False # XXX
        # XXX ^^^

        self.all_picks[pck0.id] = pck0

        if ignore_new_trigger:
            if self.verbosity>=1:
                self.logger.write("feed %s too many triggers - IGNORED\n" % \
                    pck0.id)
            return 0, []

        if self.verbosity>=1:
            self.logger.write("feed %s\n" % pck0.id)

        # test if the unassociated pick matches a known origin
        match_org = self.compare_with_known_events(pck0)
        if pck0.ignore:
            if self.verbosity>=1:
                self.logger.write("feed: %s marked ignored\n" % pck0.id)
            return 0, [] # XXX ugly, better to return matching phase

        # if a (potentially) matching origin was found...
        if match_org:
            self.logger.write("XXX XXX2 id=%s\n" % match_org.id)
            if self.verbosity>=1:
                self.logger.write("Trg %s might match evt %s\n" % \
                                (pck0.id,match_org.id))
            try:
                # ... try to improve that origin using the new pick
                # On failure this raises a LocationFailed exception
                self.perhaps_improve_origin(match_org, pck0)
                # At this point the origin has successfully been improved
                # So we now test if any of the previously not associated
                # picks matches the new origin
                # XXX Vielleicht sollte man das auslagern?
#               otime = match_org.time
#               for pck in self.all_picks.values():
#                   if  otime < pck.on < otime+1800 \
#                       and pck.id not in self.associated \
#                       and pck    not in match_org.trgs \
#                       and self.__pick_matches_origin(match_org, pck):
#                       try:
#                           self.perhaps_improve_origin(match_org, pck)
#                           self.logger.write("Belatedly added pick %s to evt %s\n" \
#                                           % (pck.id, match_org.id))
#                       except LocationFailed:
#                           pass
                            
            except LocationFailed:
                match_org = None

        if not match_org:
            new_origins = []
            trial_origins  = []

        # loop over beams to collect pick trial_origins which might
        # lead to a successful location. Redundant trial_origins may be
        # obtained that will be sorted out after the loop.

        if match_org:
                add_only=True
        else:   add_only=False

        for ib, beam in enumerate(self.beams): # feed new pick into each beam

            org = beam.feed(pck0, add_only=add_only)
            if org:
#               self.logger.write("XXX XXX3 id=%s\n" % org.id)
                trial_origins.append(org)

        if match_org:
            return 2, [ match_org ]

        # ########################################################### #

#       trial_origins = remove_associated_picks(trial_origins, self.associated)
        trial_origins = remove_outlying_picks(trial_origins)
        if len(trial_origins) > 1:
            trial_origins = remove_redundant_trial_origins(trial_origins)

        if len(trial_origins) == 0:
            return 0, []

        trial_origins = sort_trial_origins(trial_origins)

        # now for each of the found trial_origins try a location until success
        for trial_org in trial_origins[:10]:
        # XXX dangerous because we may miss small events in case of overlap
        # XXX with bigger ones.
            if len(trial_org.trgs) < self.nmin:
                # since a pick might have been removed
                continue
            try:
                org = self.locate.locate2(trial_org, autorefine=3)
                if org.rms_res > max_rms_res:
                    print "Location failed RMS=%.2f" % org.rms_res
                    continue
            except LocationFailed:
#           except:
                continue # proceed to next beam

            org.detected = time.gmtime()    # XXX
            org.status = "N"                # XXX
            new_origins.append(org)

        if len(new_origins) == 0:
            return 0, []

        new_origins = remove_outlying_picks(new_origins)

        # sort origins according to number of picks (largest first):
        tmp = [ (len(org.trgs), org) for org in new_origins ]
        tmp.sort();
        new_origins = [ org for n,org in tmp[::-1] ]

        # now, new_origins is a list of *possibly* new origins, sorted by
        # number of contributing picks in descending order (origins with
        # most picks first come first).

        new_origins2 = [ ]
        improvement = []
        # are there any existing origins that share the same set of picks?
        for i,new_org in enumerate(new_origins):
            _tmin = min( [ pck.time for pck in new_org.trgs] )
            new_org.id = _tmin.strftime("ev%y%m%d%H%M%S")
            new_org.bad = False

            is_new = True # unless we find a matching existing event

            # loop over already existing origins
            for i, org in enumerate(self.origins):
                overlap = list_overlap(new_org.trgs, org.trgs)

                if not overlap:
                    continue # fine

#               if len(new_org.trgs)-len(overlap) < self.nmin:
                if len(new_org.trgs)-len(overlap) < 3:
                    new_org.bad = True # XXX don't call it 'bad'
                    continue
                # else try without the overlapping picks, but
                # also for the existing event
# XXX handle the case where there is overlap but very different origins and
# many picks.
                is_new = False
                comp = compare_origin_quality(org, new_org, self.logger)
                if self.verbosity>=1:
                    self.logger.write("compare_origin_quality Y %s = %.4f\n" % \
                    (org.id, comp))
                if comp <= 0:
                    new_org.bad = True # XXX don't call it 'bad'
                    continue

                improvement.append( (org, new_org) )
                break

            if new_org.bad:
                continue # don't do anything else with this origin

            if is_new:
                new_org.status = "N" # this is a new event
                new_org.detected = time.gmtime()
                self.add_origin(new_org)
                new_origins2.append(new_org)
            else:
                new_org.status = "U" # this is an updated event

# XXX XXX XXX XXX XXX
        for org, new_org in improvement:
            origin_update(org, new_org)
            new_origins2.append(org)

        new_origins2 = [ org for org in new_origins2 if not org.bad ]
    
        return 3, new_origins2


    def feed(self, pck0):

        if pck0.sta not in self.staindex: # XXX urghhh!!!
            return 0, []

        status, origins = self.__feed(pck0)
        if status in (0,1):
            return status, origins

        if pck0.id not in self.all_picks: # XXX urghhh!!!
# XXX       self.logger.write("MISSING %s in all_picks @ 1" % pck0.id)
            return 0, []

        for org in origins:
#           self.logger.write("XXX XXX1 id=%s\n" % org.id)

            retry = True
            while retry:
                retry = False

                for pck_id in self.associated.keys()[:]:
                    xorg = self.associated[pck_id]
                    if not pck_id in self.all_picks:
# XXX                  self.logger.write("MISSING %s in all_picks @ 2\n" % pck_id)
                        continue # XXX This should never happen FIXME
                    xpck = self.all_picks[pck_id]

                    if xorg.id==org.id and not xpck in org.trgs:
                        try:
                            self.perhaps_improve_origin(org, xpck)
                            if self.verbosity>=1:
                                self.logger.write("Repeatedly added pick %s " \
                                           "to event %s\n" % (xpck.id, org.id))
                            retry = True
                        except LocationFailed:
                            pass

                otime = org.time
                for pck in self.all_picks.values():
                    if  otime < pck.on < otime+1800 \
                        and pck.id not in self.associated \
                        and pck    not in org.trgs \
                        and self.__pick_matches_origin(org, pck):
                        try:
                            self.perhaps_improve_origin(org, pck)
                            if self.verbosity>=1:
                                self.logger.write("Belatedly added pick %s" \
                                        " to event %s\n" % (pck.id, org.id))
                            retry = True
                        except LocationFailed:
                            pass

            if good_origin(org):
                for pck in org.trgs:
                    for arr in org.arr:
                        if arr.id == pck.id:
                            break
                    if arr.id != pck.id:
                        continue
                    if not arr.used:
                        if arr.id in self.associated:
                            if self.associated[pck.id] is org:
                                del self.associated[pck.id]
                        continue
                    if not pck.id in self.associated:
                        self.associated[pck.id] = org
            # XXX remove possibly disassociated picks

        return 2, origins


def read_beam_file(filename):
    beams = [ tuple(map(float, line.split()))
                for line in file(filename).xreadlines() ]
    beams.reverse()
    return beams

def make_eventdir(eventdir, org):
    this_eventdir = "%s/%s" % (eventdir, org.id)
    if not os.path.exists(this_eventdir):
            os.mkdir(this_eventdir)
    elif not os.path.isdir(this_eventdir):
        this_eventdir = None
    return this_eventdir

def time_stamp(t=None):
    return time.strftime("%Y/%m/%d %T UTC", time.gmtime())

def store_in_file(org, root_eventdir):

    global hashes

    this_eventdir = make_eventdir(root_eventdir, org)
    msg = print_origin_procalert(org)

    if this_eventdir:
        # dump origin to file
        f = file("%s/temp" % root_eventdir, "w")
        f.write("\n%s\n" % msg)
        f.close()
        os.rename("%s/temp" % root_eventdir,
                  "%s/last" % this_eventdir)

        # dump origin to history file
        f = file("%s/history" % this_eventdir, "a")
        tst = time_stamp()
        f.write("\n#### time is now %s\n%s\n" % (tst, msg))
        f.close()

        # dump origin to procalert file # XXX very preliminary
        f = file("%s/temp" % root_eventdir, "w")
        f.write("\n%s\n" % msg)
        f.close()
        os.rename("%s/temp" % root_eventdir,
                  "%s/procalert" % this_eventdir)

    # Seismic Handler pick file support
    f = file("%s/temp" % root_eventdir, "w")
    print_origin_seismic_handler(org, f)
    f.close()
    # XXX This is a weird hack XXX
    evtnr = int(org.time.time1970 % 86400)/90
    evtid = "%03d%02d%02d%03d" % \
            (org.time.yr%1900, org.time.mo, org.time.day, evtnr)
    fname1 = "%s/shm_%s.evt" % (this_eventdir, evtid)
    fname2 = "%s/shm_%s.evt" % (root_eventdir, evtid)
    os.rename("%s/temp" % root_eventdir, fname1)
    # create a symlink for backwards compatibility
    if not os.path.exists(fname2):
        os.symlink(fname1.replace(root_eventdir,"").lstrip("/"), fname2)

def store_in_db(org, db):

    global autoloc # FIXME

    associations = []
    picks = []
    magnitudes = []
    for p in org.arr:
        asso = Association(origin_id=org.id, pick_id=p.id,
                    phase="P", residual=p.res, used=p.used)
        associations.append(asso)
        picks.append(autoloc.all_picks[p.id]) # XXX

    mb, amp_mb, per_mb = compute_magnitude_mb(org)
    ml, amp_ml         = compute_magnitude_ml(org)
    for p in org.arr:
        try:
            _mb = mb[p.sta]
            m = StationMagnitude(org.id, p.net, p.sta, "mb", _mb)
            magnitudes.append(m)
        except KeyError: pass

        try:
            _ml = ml[p.sta]
            m = StationMagnitude(org.id, p.net, p.sta, "ML", _ml)
            magnitudes.append(m)
        except KeyError: pass

    db.put_origins([ loc2org(org) ])
    db.put_associations(associations)
    db.put_picks(picks)
    db.put_sta_magnitudes(magnitudes)


from getopt import getopt, GetoptError

class Config: pass

config = Config()
config.tmpdir    = "/tmp"
config.logdir = None
config.verbosity = 0
config.realtime  = 0

usage_info = "USAGE\n"
beamfile = stafile = None
trigfile = sys.stdin
eventdir = None
db_config = None
autoloc = None

def usage(exitcode=0):
    sys.stderr.write(usage_info)
    sys.exit(exitcode)

def positive_float(arg):
    arg = float(arg)
    if arg<0:
        raise TypeError, "argument must not be negative"
    return arg

def sig_handler(signum, frame):
    global autoloc

    if signum==15:
        if config.verbosity>=1:
            logger.write("Caught SIGTERM - exiting\n")
        del autoloc
        sys.exit(0)

signal.signal(signal.SIGTERM, sig_handler)
                        
try: # parse command line arguments
    opts, input_files = getopt(sys.argv[1:], "b:f:l:d:e:s:T:rv")
except GetoptError:
    usage(exitcode=2)

for flag, arg in opts:
    if   flag == "-b":  beamfile = arg
    elif flag == "-f":  trigfile = arg
    elif flag == "-l":  config.logdir = arg
    elif flag == "-T":  config.tmpdir = arg
    elif flag == "-e":  eventdir = arg
    elif flag in ("-d", "--db-conf"):
        db_config = arg
    elif flag == "-s":  stafile = arg
    elif flag == "-r":  config.realtime = True
    elif flag == "-v":  config.verbosity += 1
    elif flag == "-h":  usage()

if not beamfile or not stafile:
    usage(2)

if not config.logdir and eventdir:
    config.logdir = eventdir

logger  = Logger(os.path.join(config.logdir, "autoloc-log-"))

if db_config:
    if config.verbosity >= 2:
        logger.write("reading db config from file '%s'\n" % db_config)
    db_config = seiscomp.event_db.get_config(db_config)
    if config.verbosity >= 3:
        logger.write("%s\n" % str(db_config))
    db = seiscomp.event_db.EventDB(db_config, logger=logger,
                                   verbosity=config.verbosity)
else:
    db = None

stations = read_station_inf(stafile)
beams    = read_beam_file(beamfile)
autoloc  = AutoLoc(beams, stations, config, logger, db)

pwd = os.getcwd()

iii = 0

if config.realtime: input = PickInput(trigfile, forever=True,  timeout=5)
else:               input = PickInput(trigfile, forever=False, timeout=5)

hashes = {}

for obj in input:

    if type(obj) is Pick: # obj may also be e.g. None after timeout
        pck = obj 

        try:
            peri_mb, ampl_mb = pck.peri_mb, pck.ampl_mb
        except:
            # The pick is required to have mb attributes.
            # If not (yet) -> go on to next pick
            continue


        status, origins = autoloc.feed(pck)
    else:
        # autoloc.keep_up()
        origins = None

    if status>0: 

#       pck_ids = autoloc.associated.keys()
#       pck_ids.sort()
#       autoloc.logger.write("\nASSOCIATIONS:\n")
#       for id in pck_ids:
#           autoloc.logger.write("ASSO %16s %-40s\n" % (autoloc.associated[id].id, id))
#       autoloc.logger.write("\n")
        
        for org in origins:
#           logger.write("XXX YYY id=%s\n" % org.id)
            # discard origins with too large a confidence region
            if org.confreg_semmaj > max_con_reg:
                # XXX make this configurable, perhaps even by making the
                # XXX threshold depend on the average station distance
                continue

            msg = print_origin_procalert(org) # XXX also sets org.md5hash

            # XXX move to Autoloc.feed()
            # compute hash to avoid sending duplicate origins
            md5 = org.md5hash # XXX set by print_origin_procalert()
            if org.id in hashes:
                if hashes[org.id] == md5:
                    continue
            hashes[org.id] = md5

            logger.write("\n%s\n" + msg)
            logger.write("\n%s\n" + print_summary_line(org))

            if eventdir:
                store_in_file(org, eventdir)

            if db:
                store_in_db(org, db)

    iii += 1
    if iii > 500: # every once in a while clean-up picks

        autoloc.truncate() # XXX move into Autoloc.feed()
        iii = 0
