import  sys
from    seis.types import Trace
from    seis import _ah

def _file_r(f):
    # Tries to make a readable file from object f.
    if f is None:
        f = sys.stdin
    if type(f) is str:
        f = file(f)
    if type(f) is not file:
        raise TypeError, "cannot read AH data from '%s'" % str(type(f))
    assert 'r' in f.mode
    return f

def _file_w(f):
    # Tries to make a writable file from object f.
    if f is None:
        f = sys.stdout
    if type(f) is str:
        f = file(f, "w")
    if type(f) is not file:
        raise TypeError, "cannot read AH data from '%s'" % str(type(f))
    assert 'w' in f.mode
    return f

def read(f=None, n=999999999):

    f = _file_r(f)

    if n == 1:
        return _ah._read(f)
    else:
        traces = []
        for i in xrange(n):
            try:
                traces.append(_ah._read(f))
            except EOFError:
                break
        return traces


def write(trc, f=None):

    if isinstance(trc, Trace):
        traces = [ trc ]
    elif isinstance(trc, list):
        traces = trc

    f = _file_w(f)

    for trc in traces:
        _ah._write(trc, f)


class InputFile(file):

    """ Experimental - don't use!!!"""

    def __init__(self, name):
        raise Exception, "Don't use InputFile (it is experimental)"
        file.__init__(self, name, "rb", 1024)

    def read(self, n=1): # XXX
        if n > 1:
            return [ _ah._read(self) for i in xrange(n)]
        return _ah._read(self)

    def next(self):
        return self.read(1)


class OutputFile(file):

    """ Experimental - don't use!!!"""

    def __init__(self, name):
        raise Exception, "Don't use OutputFile (it is experimental)"
        file.__init__(self, name, "wb", 1024)

    def write(self, trc):
        if isinstance(trc, Trace):
            traces = [ trc ]
        elif isinstance(trc, list):
            traces = trc
        else:
            raise TypeError, "cannot write '%s'" % type(trc)

        _ah._write(trc, self)


def _InputFileList(files):

    if not files:
        files = [ sys.stdin ]
    if not isinstance(files, list):
        files = [ files ]

    for f in files:
        yield _file_r(f)  # yield the *open* file


class Input(object):
    """
    Input iterator class for AH data

    Iterates over the records in an open file-like object until EOF
    is reached. Particularly useful in 'for'-loops:

        import seis.ah

        for record in seis.ah.Input(f):
            ...

    or, if the input consists of tuples:

        for z,n,e in seis.ah.Input(f, n=3):
            ...

    where n is the length of the tuple.
    """

    def __init__(self, source=sys.stdin, n=1):
        """
        The 'source' to read AH records from can be:
            - an open, readable, file-like object
	    - a name of a file to read from
	    - a list containing any of the above
        """
        assert n >= 1
        self.__n  = n

        if not source:
            # includes the case of 'source' being an empty list
            source = [ sys.stdin ]

	self.__files = _InputFileList(source)
	self.__f     = self.__files.next()

#   def __del__(self):
#       self.__f.close()
#   ## closing may not be appropriate, e.g. for stdin

    def __getitem__(self, index):
        while True:
            try:
                if self.__n > 1:
                        trc = [ _ah._read(self.__f) for i in xrange(self.__n)]
                        trc = tuple(trc)
                else:   trc = _ah._read(self.__f)
                return trc

            except EOFError:
                # see whether there is another file in the list
                try:
                    self.__f = self.__files.next()
                except StopIteration:
                    raise IndexError
