from Numeric import sin, cos, arccos, arcsin, pi

# TODO:
#
# * implement 'delazi' etc. in C/Pyrex
# * allow delazi2 to accept (and return) lists of objects

from seis._numeric import delazi, delazi_many, delandaz2coord, scxsc, scdraw

#def delazi(lat1, lon1, lat2, lon2):
#    """
#    -> del,az1,az2
#
#    Computes the distance and the two azimuts between
#    two points on a sphere in geographical coordinates.
#    
#    del is the distance between 1 and 2
#    az1 is the azimuth of 2 at 1
#    az2 is the azimuth of 1 at 2
#
#    all angles in degrees!!!
#    """
#
#    if lat1==lat2 and lon1==lon2: return 0.,0.,0.
#    pi180, ip180 = pi/180., 180./pi
#
#    # calculate delta and the two azimuths
#    a   =  (90.-lat2) * pi180
#    b   =  (90.-lat1) * pi180
#    gam = (lon1-lon2) * pi180
#    cosa,cosb,sina,sinb = cos(a),cos(b),sin(a),sin(b)
#    c   = arccos(cosa*cosb+sinb*sina*cos(gam))
#    cosc,sinc = cos(c),sin(c)
#    xx = (cosa-cosb*cosc)/(sinb*sinc)
#    print xx,cosa-cosb*cosc,sinb*sinc
#    az1 = arccos(xx) * ip180
#    xx = (cosb-cosa*cosc)/(sina*sinc)
#    print xx,cosb-cosa*cosc,sina*sinc
#    az2 = arccos(xx) * ip180
#    delta = c*ip180
#
#    if sin(gam)<0.: az2 = 360.-az2
#    else:           az1 = 360.-az1
#
#    return delta, az1, az2

def delazi2 (pt1, pt2):
    """
    -> del,az1,az2

    Like delazi, but takes as arguments objects, each of which must
    have both 'lat' and 'lon' attributes. This can be convenient
    e.g. to compute the distance between an event and a station.
    """
    if  hasattr(pt1, 'lat') and hasattr(pt1, 'lon') and \
	hasattr(pt2, 'lat') and hasattr(pt2, 'lon'):
	return delazi(pt1.lat, pt1.lon, pt2.lat, pt2.lon)
    raise ValueError, "can't compute delazi2"

def xandaz2coord(x, az, lat0, lon0):
    """
    -> lat, lon

    Returns the coordinates of the point which is at an azimuth of 'az'
    and a distance of 'x' km as seen from the point (lat0,lon0).
    """
    # x in km, az,lat,lon in deg
    return delandaz2coord(x/111.195, az, lat0, lon0)

#def delandaz2coord(d, az, lat0, lon0):
#    """
#    -> lat, lon
#
#   Returns the coordinates of the point which is at an azimuth of 'az'
#   and a distance of 'd' as seen from the point (lat0,lon0).
#   """
#   # x in km, az,lat,lon in deg
#   _lat, az, d = pi*(90.-lat0)/180., az*pi/180., d*pi/180.
#   _latx = arccos(cos(_lat)*cos(d)+sin(_lat)*sin(d)*cos(az))
#   _lonx = arcsin(sin(d)*sin(az)/sin(_latx))
#   return 90. - 180.*_latx/pi, lon0 + 180.*_lonx/pi




def inrange_del(origin, objects, dmin, dmax):
    """
    -> list or dict (depending on whether 'objects' is a list or a dict)
    
    Returns a list/dict of objects within the distance range from dmin to
    dmax (in degrees) relative to origin. Both 'origin' and the elements
    of the list 'objects' must have the attributes 'lat' and 'lon',
    containing the latitude and longitude, respectively. Convenient e.g.
    to quickly find the stations within a certain distance range relative
    to en event or viceversa.
    """

    if isinstance(objects, list):
        inrange = []
        for obj in objects:
            delta, az1, az2 = delazi2(origin, obj)
            if dmin <= delta <= dmax:
                inrange.append(obj)
    elif isinstance(objects, dict):
        inrange = {}
        for key in objects:
            delta, az1, az2 = delazi2(origin, objects[key])
            if dmin <= delta <= dmax:
                inrange[key] = objects[key]
    else:
        raise TypeError, 'objects: illegal type %s' % str(type(objects))

    return inrange

