from Numeric import array, average, sin, cos, sqrt, exp, sum, arcsin, arctan2

def sorted_by_time(thelist):
    # Returns a shallow copy of the list sorted by time.
    # Each list member must have a 'time' attribute
    tmplist = [ (item.time, item) for item in thelist ]     # decorate
    tmplist.sort()                                          # sort
    thelist = [ item for t, item in tmplist ]               # undecorate
    return thelist

def pol_weight(angle, width):
    # empirical weight function, 180-degree periodicity
    return exp(-sin(angle*pi/180.)**2/(2*width/180.)**2)/width

def azi_inc_to_vector (azi, inc):
    # computes a unit vector pointing into direction of (azi, inc)
    return sin(azi)*cos(inc), cos(azi)*cos(inc), sin(inc)


def time_average(times):
    t0 = times[0]
    return t0 + average(array([ float(t-t0) for t in times ]))

def list_contains_list(lista, listb):
    # checks if ALL items of listb are also contained in lista
    for item in listb:
        if not item in lista:
            return False
    return True

def list_overlap(lista, listb):
    # returns the list items that are members of both lists
    return [ item for item in lista if item in listb ]


def average_origin(origins):

    lat     = array([org.lat for org in origins])*pi/180
    lon     = array([org.lon for org in origins])*pi/180
    x, y, z = sum(sin(lon)*cos(lat)), sum(cos(lon)*cos(lat)), sum(sin(lat))
    n       = sqrt(x*x+y*y+z*z)
    x, y, z = x/n,y/n,z/n
    lat     = arcsin(z)*180/pi
    lon     = arctan2(x,y)*180/pi

    dep     = average([org.dep for org in origins])
    t0      = origins[0].time
    tim     = average([org.time-t0 for org in origins])+t0

    return (tim, lat, lon, dep)

