#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

static int les_linje();
static void howto (void); 
static int params(int argcount, char **argvec);

int    printarg = 1;
int    control = 0;
int    sw_top = 0;
int    sw_fna = 0;
int    sw_coo = 0;
int    sw_epi = 0;
int    sw_sta = 0;
int    sw_gvv = 0;
int    sw_sub = 0;

int    xmi;
int    xma;
int    ymi;
int    yma;

float  longitude;
float  latitude;
  
char   filename[300];                            // map filename
char   ttitle[300];                              // top title   
char   btitle[300];                              // bottom title
char   top[300];
char   dummy[300];
char   du[300];
char   coordinates[300];                         // xmin,xmax,ymin,ymax
int main(int argc, char **argv)
{
  FILE *sf;
  FILE *in;
  FILE *fp;
  char     *topdir_html = 0;                     // top directory HTML pages
  char     *topdir_rtquake = 0;                  // top directory RTQUAKE  
  char     filepath[300];                        // path to GMT input data
  char     filepath2[300];                       // path to GMT input data, last location
  char     filepath3[300];                       // path to ALL_LOC.txt 
  char     gmtworkpath[300];                     // path to GMT work directory

  char     syscmd[300];
  char     record[300];
  char     lastli[300];
  char     tid1[20];
  char     date[20];  
  char     tid[40];
  char     lat1[10];
  char     lat2[20];
  char     lon1[20];
  char     lon2[20];
  char     mw1[10];
  char     mw2[10];
  char     ml1[10];
  char     ml2[10];
  char     subtitle[200];

  char     top2[300];
  int      ret;
  int      br;
  int      i;
  
  float    newlat1;
  float    newlat2;

//-------------------------------------------------------------------------
// Process command line parameters
//-------------------------------------------------------------------------
  if (params(argc, argv) < 0)
  {
    printf("RTGMT: Parameter processing failed\n\n");
    printf("           '-h' for detailed help\n");
    exit(0);
  }  
//------------------------------------------------------------------------
// set path to HTML_TOP as default
//------------------------------------------------------------------------
  topdir_html = (char*)getenv("HTML_TOP");
  if (topdir_html)
  {
    printf("RTGMT: HTML_TOP.........................: %s\n", topdir_html);
  }else{
    printf("RTGMT: HTML_TOP not defined\n");
    exit(0);      
  }
//------------------------------------------------------------------------
// set path to RTQUAKE_TOP as default
//------------------------------------------------------------------------
  topdir_rtquake = (char*)getenv("RTQUAKE_TOP");
  if (topdir_rtquake)
  {
    printf("RTGMT: RTQUAKE_TOP......................: %s\n", topdir_rtquake);
  }else{
    printf("RTGMT: RTQUAKE_TOP not defined\n");
    exit(0);      
  }   
  sprintf(filepath,"%s/map/GMT_EPI0.txt",topdir_html); 
  sprintf(filepath2,"%s/map/GMT_LAST_LOC.txt",topdir_html);
  sprintf(filepath3,"%s/map/ALL_LOC.txt",topdir_html);
//-------------------------------------------------------------------------------------------  

  


  
  if(sw_top == 0)                                // read top title from file
  {
    in = fopen(filepath2,"r");
    if(in == NULL)
    {
      printf("Can't open %s\n",filepath);
      exit(0);
    }
    br = les_linje(in,record);
    fclose(in);
    printf("1  %s\n",record);

    for(i=0;i<br;i++)
      top[i]=record[i+51];
    printf("2  %s\n",top);
  
    sprintf(syscmd,"echo %1c%s%1c  | iconv -f utf-8 -t ascii//TRANSLIT",0x22,top,0x22);
    printf("3  %s\n",syscmd);
    
    fp = popen(syscmd,"r");
    if(fp==NULL)
    {
      printf("Failed to run command\n");
      exit(0);
    }
    if(fgets(top2,sizeof(top2)-1,fp) != NULL)
      ret=0;
  }else{
    sprintf(top2,"%s",top);
    printf("%s\n",top2);
  }

//-------------------------------------------------------------------------------------------
  
  if(sw_fna == 1)                                // Map filename (default Plot.stn
  {
      
  }else{
    sprintf(filename,"Plot_stn");
      
  }

//--------------------------------------------------------------------------------------------
  
  if(sw_coo == 1)
  {
    printf("%s\n",coordinates);
    sscanf(coordinates,"%d %d %d %d",&xmi,&xma,&ymi,&yma);
//    exit(0);
  }else{
    in = fopen(filepath2,"r");                   // find center coordinates last event
    if(in == NULL)
    {
      printf("Can't open %s\n",filepath);
      exit(0);
    }
    br = les_linje(in,record);
    fclose(in);
    printf("%s\n",record);
    sscanf(record,"%19c%f%f",dummy,&longitude,&latitude);
    printf("%f8.2    %f8.2\n",longitude,latitude);
    xmi = (int)longitude -3;
    xma = (int)longitude +3;
    ymi = (int)latitude -4;
    yma = (int)latitude +4;
    
    newlat1 = latitude - (300.0/6373) * (180/3.14);
    newlat2 = latitude + (300.0/6373) * (180/3.14); 
    
    ymi = (int)newlat1;
    yma = (int)newlat2;
  }

  printf("xmi:%d\n",xmi);
  printf("xma:%d\n",xma);
  printf("ymi:%d\n",ymi);
  printf("yma:%d\n",yma); 
  printf("%d\n",(xmi+xma)/2);
//-----------------------------------------------------------------------------
// Create subtitle and position
//-----------------------------------------------------------------------------
  if(sw_sub == 0)
  {
    in = fopen(filepath3,"r");
    if(in == NULL)
    {
      printf("Can't open %s\n",filepath);
      exit(0);
    }
    
    for(;;)
    {
      br = les_linje(in,record);
      if(br != 0)
      {
        sprintf(lastli,"%s\n",record);
      }else{
        printf("%s\n",lastli);
        sscanf(lastli,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s",tid1,date,tid,du,lat1,du,lat2,du,lon1,du,lon2,du,mw1,du,mw2,du,ml1,du,ml2);
        printf("%s %s %s %s %s %s %s %s %s %s %s\n",tid1,date,tid,lat1,lat2,lon1,lon2,mw1,mw2,ml1,ml2);
        sprintf(subtitle,"%6.2f %6.2f BC %s %s %s %s %s %s %s %s %s %s %s\n",(float)((xmi+xma)/2),(float)(ymi-0.5),tid1,date,tid,lat1,lat2,lon1,lon2,mw1,mw2,ml1,ml2);  
        printf("%s\n",subtitle);      
        break;
      }
    }
  }
//--------------------------------------------------------------------------------------------
// Generate script
//-------------------------------------------------------------------------------------------- 
  sprintf(gmtworkpath,"%s/rtqgmt/rtqgmt.sh",topdir_rtquake);
  printf("%s\n",gmtworkpath);
  sf = fopen(gmtworkpath,"w");
  if(sf == NULL)
  {
    printf("Can't open rtqgmt.sh\n");
    exit(0);
  }
  


  fprintf(sf,"#!/bin/bash\n");
  fprintf(sf,"file=%s.ps\n",filename);  
  fprintf(sf,"rm -f *.grd\n");

  fprintf(sf,"grdfile=%s/rtqgmt/DATA/newbathy.grd\n",topdir_rtquake);  
//  fprintf(sf,"grdfile=$HOME/GEBCO/newbathy.grd\n");
//  fprintf(sf,"grdfile=$HOME/GEBCO/Bathymetry_GEBCO_30s_world.grd\n");
  
  fprintf(sf,"xmin=%d\n",xmi);
  fprintf(sf,"xmax=%d\n",xma);
  fprintf(sf,"ymax=%d\n",yma);
  fprintf(sf,"ymin=%d\n",ymi);  

  fprintf(sf,"midlon=`gmt gmtmath -Q $xmin $xmax ADD 2 DIV =`\n");
  fprintf(sf,"midlat=`gmt gmtmath -Q $ymin $ymax ADD 2 DIV =`\n");

  fprintf(sf,"R=%1c-R$xmin/$xmax/$ymin/$ymax%1c\n",0x22,0x22);

  fprintf(sf,"J=%1c-JM$midlon/$midlat/16c%1c\n",0x22,0x22);

  fprintf(sf,"gmt grdcut  $grdfile  $R -Gtmpf.grd\n");

  fprintf(sf,"gmt grdinfo tmpf.grd\n");

  fprintf(sf,"gmt makecpt -Cglobe -T-8500/8500/50 -Z > cptfile2.cpt\n");

  fprintf(sf,"top2=`echo %s`\n",syscmd);
  
  fprintf(sf,"gmt psbasemap -B1g1 -B+t%c%s%c $J $R -K -P > $file\n",0x22,top2,0x22);   

  fprintf(sf,"gmt grdgradient tmpf.grd -Nt1 -A225 -Gtmpgrd.grd\n");

  fprintf(sf,"gmt grdimage $J $R  tmpf.grd  -Itmpgrd.grd  -O -K  -Ccptfile2.cpt >> $file\n");

  fprintf(sf,"gmt pscoast $J -B $R -Dh -N1/1.00p,- -K -O -W >> $file\n");

  fprintf(sf,"sizecoef=.2\n");

//# Variable used inside loop to adjust position of text-label
  fprintf(sf,"adjust=-0.2\n");
  if(sw_sta == 0)
  {
    fprintf(sf,"while read line\n");
    fprintf(sf,"do\n");
    fprintf(sf,"  lon=`echo $line | gawk '{print $1}' `\n");
    fprintf(sf,"  lat=`echo $line | gawk '{print $2}'`\n");
    fprintf(sf,"  sym=`echo $line | gawk '{print $3}' `\n");
    fprintf(sf,"  col=`echo $line | gawk '{print $4}' `\n");
    fprintf(sf,"  cof=`echo $line | gawk -v var=$sizecoef '{print $5*var}' `\n");
    fprintf(sf,"  stn=`echo $line | gawk '{print $6}' `\n");
//  fprintf(sf,"echo $lon $lat $sym $col $cof\n");
 
    fprintf(sf,"echo $lon $lat $sym$cof | gmt psxy $J $R -O -K -S -W  -G$col   >> $file\n");

//# Compute new longitude for text-label
    fprintf(sf,"lon2=$(echo %1c$lon + $adjust%1c|bc)\n",0x22,0x22);

    fprintf(sf,"echo $lon2 $lat $stn    | gmt pstext $J $R -O -F+f12p,Helvetica-Bold+jRB -K >> $file\n");

//    fprintf(sf,"done < %1cstations%1c\n",0x22,0x22);
    
    fprintf(sf,"done < %1c%s/rtqgmt/stations%1c\n",0x22,topdir_rtquake,0x22);    
    
  }
  
  fprintf(sf,"gmt pstext -R -J -N -K -O -V -F+f15,1,blue+j << EOF >> $file\n");
  fprintf(sf,"%s",subtitle);
  fprintf(sf,"EOF\n");
  
  
  if(sw_epi == 0)
  {
//    fprintf(sf,"gmt psxy  $R $J $B -O %s -Wfaint -i6,7,10,9s0.2 -h0 -Scc -Cquakes.cpt -K >> $file\n",filepath);  
//    fprintf(sf,"gmt psxy  $R $J $B -O %s -Wfaint -i6,7,10,9s0.2 -h0 -Sac -Cquakes.cpt >> $file\n",filepath2); 
    
    fprintf(sf,"gmt psxy  $R $J $B -O %s -Wfaint -i6,7,10,9s0.2 -h0 -Scc -C%s/rtqgmt/quakes.cpt -K >> $file\n",filepath,topdir_rtquake);  
    fprintf(sf,"gmt psxy  $R $J $B -O %s -Wfaint -i6,7,10,9s0.2 -h0 -Sac -C%s/rtqgmt/quakes.cpt >> $file\n",filepath2,topdir_rtquake);     
    
  }else{
// Dummy statement to terminate the ps-file 
//    fprintf(sf,"gmt psxy  $R $J $B -O %s -Wfaint -i6,7,10,9s0.2 -h1 -Sac -Cquakes.cpt >> $file\n",filepath2);   
    
    fprintf(sf,"gmt psxy  $R $J $B -O %s -Wfaint -i6,7,10,9s0.2 -h1 -Sac -C%s/rtqgmt/quakes.cpt >> $file\n",filepath2,topdir_rtquake);    
    
  }
  fprintf(sf,"outeps=`echo $file | sed 's/ps/eps/g'`\n");

  fprintf(sf,"eps2eps  $file $outeps\n");

  fprintf(sf,"gmt psconvert $outeps -Tj -P\n");

  printf("sw_gvv: %d\n",sw_gvv);
  if(sw_gvv == 0)
  {
    fprintf(sf,"gv $file\n");
  }
  fclose(sf);
  
  sprintf(syscmd,"chmod +x %s/rtqgmt/rtqgmt.sh\n",topdir_rtquake);  
  ret = system(syscmd);
  sprintf(syscmd,"%s/rtqgmt/rtqgmt.sh",topdir_rtquake);
  ret = system(syscmd);
  
  
  
//  sprintf(syscmd,"chmod +x rtqgmt.sh\n");
//  ret = system(syscmd);
//  sprintf(syscmd,"./rtqgmt.sh");
//  printf("%s\n",syscmd);
//  ret = system(syscmd);
}
int les_linje(FILE *streamfp,char linje[])
{
  int l;
  int br;
  br=0;
  for(l=0;l<200;l++)
  {
    linje[l]=fgetc(streamfp);
    if(linje[l] == 0xa && l == 0)
    {
      br=2;
      break;
    }
    if(linje[l] == 0xa && l > 0)
    {
      linje[l]='\0';
      break;        
    }
    if(linje[l] == EOF)
    {
//      printf("End of file. File contains: %2d items\n",i);

      br=1;
      return(l);
    }
  }
  return(l);
}                        // les_linje
//---------------------------------------------------------------------------
// read command line parameters
//---------------------------------------------------------------------------
int
params(int argcount, char **argvec)
{
  int  optind;
  int  error = 0;
  int  code;
  char tm[80];  
  char streamsfull[80]; 
  char *multiselect = 0;
  char *selectors   = 0;

  code = 0;
  if (argcount <= 1)
    error++;

  /* Process all command line arguments */
  for (optind = 1; optind < argcount; optind++)
  {
    if (strcmp (argvec[optind], "-h") == 0)
    {
      howto();
      exit (0);
    }

    else if (strcmp (argvec[optind], "-tt") == 0)         // top title
    {
      strcpy(top, argvec[++optind]);
      sw_top = 1;
    }
    

    else if (strcmp (argvec[optind], "-fn") == 0)         // top title
    {
      strcpy(filename, argvec[++optind]);
      sw_fna = 1;
    }

    else if (strcmp (argvec[optind], "-co") == 0)         // xmin,xmax,ymax,ymin
    {
      strcpy(coordinates, argvec[++optind]);
      sw_coo = 1;
    }   
    

    else if (strcmp (argvec[optind], "-stat") == 0)         // no stations
    {
      sw_sta = 1;
    }

    else if (strcmp (argvec[optind], "-epi") == 0)         // no locations
    {
      sw_epi = 1;
      printf("sw_epi: %d\n",sw_epi);
    }
    
    else if (strcmp (argvec[optind], "-gv") == 0)         // no preview
    {
      sw_gvv = 1;
    }
    else if (strcmp (argvec[optind], "-st") == 0)         // no subtitle
    {
      sw_sub = 1;
    }
  }
}
    
//--------------------------------------------------------------------------
// howto:
// howto message and exit.
//--------------------------------------------------------------------------
static void
howto(void)
{
  
  printf("Options:\n");
  printf(" -h              show this message\n");
  printf(" -tt     text    Top title.\n");
  printf(" -fn     text    Map filename, default:Plot_stn \n");
  printf(" -co     text    Coordinates: xmin,xmax,ymin,ymax.\n");
  printf(" -stat           no stations\n");
  printf(" -epi            no locations\n");
  printf(" -gv             no preview with gv\n");
  printf(" -st             no subtitle below map\n");
}    

